"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const AnimateEventDetail_1 = require("../../model/AnimateEventDetail");
const AnimationFinishedEventDetail_1 = require("../../model/AnimationFinishedEventDetail");
const Events_1 = require("../../model/Events");
const AnimationLoopStyle_1 = require("./AnimationLoopStyle");
const States_1 = require("../../model/States");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class AnimationFeature extends aframe_typescript_boilerplate_1.AbstractFeature {
    constructor(controller, entity) {
        super("animation-controller", controller, entity);
        this.clips = new Map();
        this.actions = new Map();
        this.stateSystemController = controller.getSystemController("state");
        this.movementState = this.stateSystemController.getState(this.entity, States_1.States.STATE_MOVEMENT);
    }
    init() {
        this.addEventListener(Events_1.Events.EVENT_MODEL_LOADED, () => {
            this.initAnimation();
        });
        this.addEventListener(Events_1.Events.EVENT_ANIMATE_BEGIN, (detail) => {
            this.beginAnimation(detail.clipName, detail.style, detail.repetitions);
        });
        this.addEventListener(Events_1.Events.EVENT_ANIMATE_END, (detail) => {
            this.endAnimation(detail.clipName);
        });
    }
    update(data, oldData) {
    }
    remove() {
    }
    pause() {
    }
    play() {
    }
    tick(time, timeDelta) {
        if (this.mixer) {
            if (this.actions.has("walking")) {
                const walkingAction = this.actions.get("walking");
                // TODO 0.3 scale factor and 1.9 animation walk speed are model specific
                const walkingAnimationNormalSpeed = this.entity.getAttribute("scale").x / 0.4 * 2; // 1 m/s
                const entityActualSpeed = this.movementState.distanceDelta / this.movementState.timeDeltaSeconds;
                const timeScale = this.movementState.facing * entityActualSpeed / walkingAnimationNormalSpeed;
                walkingAction.setEffectiveTimeScale(timeScale);
            }
            this.mixer.update(timeDelta / 1000);
        }
    }
    initAnimation() {
        const mesh = this.entity.getObject3D('mesh');
        const clips = mesh.animations;
        if (!clips) {
            return;
        }
        if (clips.length == 0) {
            return;
        }
        this.mixer = new three_1.AnimationMixer(mesh);
        this.mixer.addEventListener('finished', (event) => {
            this.onAnimationFinished(event.action._clip.name);
        });
        this.clips.clear();
        clips.forEach(clip => {
            this.clips.set(clip.name.toLocaleLowerCase(), clip);
        });
        this.actions.clear();
        this.dispatchEvent(Events_1.Events.EVENT_ANIMATE_BEGIN, new AnimateEventDetail_1.AnimationEventDetail("dance", AnimationLoopStyle_1.AnimationLoopStyle.LOOP_REPEAT, 0));
    }
    onAnimationFinished(clipName) {
        this.dispatchEvent(Events_1.Events.EVENT_ANIMATION_FINISHED, new AnimationFinishedEventDetail_1.AnimationFinishedEventDetails(clipName));
        this.actions.delete(clipName);
    }
    beginAnimation(clipName, style, repetitions) {
        // If clipNameOriginal is walking and walking does not exist in clips then switch to clipName walk.
        const clipNameFinal = clipName != "walking" ? clipName : (this.clips.has(clipName) ? clipName : "walk");
        if (!this.mixer) {
            return;
        }
        if (!this.clips.has(clipNameFinal)) {
            return;
        }
        const clip = this.clips.get(clipNameFinal);
        if (clip) {
            const action = this.actions.has(clipName) ? this.actions.get(clipName) : this.mixer.clipAction(clip);
            if (style == AnimationLoopStyle_1.AnimationLoopStyle.ONCE) {
                action.setLoop(three_1.LoopOnce, repetitions);
            }
            else if (style == AnimationLoopStyle_1.AnimationLoopStyle.LOOP_REPEAT) {
                if (repetitions > -1) {
                    action.setLoop(three_1.LoopRepeat, repetitions);
                }
            }
            else {
                action.setLoop(three_1.LoopPingPong, repetitions);
            }
            action.reset();
            action.fadeIn(0.25);
            action.play();
            this.actions.set(clipName, action);
        }
    }
    endAnimation(clipName) {
        if (!this.actions.has(clipName)) {
            return;
        }
        const action = this.actions.get(clipName);
        action.fadeOut(0.25);
        this.dispatchEvent(Events_1.Events.EVENT_ANIMATION_FINISHED, new AnimationFinishedEventDetail_1.AnimationFinishedEventDetails(clipName));
        setTimeout(() => {
            if (this.actions.has(clipName) && this.actions.get(clipName).weight == 0) {
                action.stop();
            }
        }, 1000);
    }
}
exports.AnimationFeature = AnimationFeature;
//# sourceMappingURL=AnimationFeature.js.map