"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DeviceSlot_1 = require("../model/DeviceSlot");
const Slot_1 = require("../model/Slot");
const Button_1 = require("../model/Button");
const xml_js_1 = require("xml-js");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
const aframe_typescript_boilerplate_2 = require("aframe-typescript-boilerplate");
const CodeRealityComponentController_1 = require("../../../component/CodeRealityComponentController");
class KeyboardAndMouseDevice extends CodeRealityComponentController_1.CodeRealityComponentController {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.movementForwardKey = 'w';
        this.movementBackwardKey = 's';
        this.movementLeftKey = 'a';
        this.movementRightKey = 'd';
        this.jumpKey = ' ';
        this.rightKey = 'ArrowRight';
        this.leftKey = 'ArrowLeft';
        this.upKey = 'ArrowUp';
        this.downKey = 'ArrowDown';
        this.pointerLock = false;
        this.lastWheelYTimeMillis = 0;
        this.interface.setDevice(DeviceSlot_1.DeviceSlot.KEYBOARD_AND_MOUSE, this);
    }
    init() {
        //console.log(this.componentName + " init");
        aframe_typescript_boilerplate_2.addDocumentEventListener("pointerlockchange", (detail) => {
            this.pointerLock = document.pointerLockElement != null;
        });
        window.addEventListener('keydown', (e) => {
            if (this.interface.isUiFocus()) {
                return;
            }
            this.onKeyDown(e.key);
        });
        window.addEventListener('keyup', (e) => {
            if (this.interface.isUiFocus()) {
                return;
            }
            this.onKeyUp(e.key);
        });
        this.entity.sceneEl.addEventListener('mousedown', (e) => {
            if (!this.pointerLock) {
                return;
            }
            if (this.interface.isUiFocus()) {
                return;
            }
            if (e.button == 0) {
                this.interface.buttonDown(this, Slot_1.Slot.PRIMARY, Button_1.Button.TRIGGER);
            }
            if (e.button == 1) {
                this.interface.buttonDown(this, Slot_1.Slot.PRIMARY, Button_1.Button.MENU);
            }
            if (e.button == 2) {
                this.interface.buttonDown(this, Slot_1.Slot.PRIMARY, Button_1.Button.GRIP);
            }
        });
        this.entity.sceneEl.addEventListener('mouseup', (e) => {
            if (!this.pointerLock) {
                return;
            }
            if (e.button == 0) {
                this.interface.buttonUp(this, Slot_1.Slot.PRIMARY, Button_1.Button.TRIGGER);
            }
            if (e.button == 1) {
                this.interface.buttonUp(this, Slot_1.Slot.PRIMARY, Button_1.Button.MENU);
            }
            if (e.button == 2) {
                this.interface.buttonUp(this, Slot_1.Slot.PRIMARY, Button_1.Button.GRIP);
            }
        });
        window.addEventListener("paste", (event) => {
            if (this.interface.isUiFocus()) {
                return;
            }
            if (this.pointerLock) {
                for (const item of event.clipboardData.items) {
                    const itemTyped = item;
                    if (itemTyped.kind == "string") {
                        itemTyped.getAsString((pasteString) => {
                            try {
                                const pasteXml = xml_js_1.js2xml(xml_js_1.xml2js(pasteString));
                                if (!pasteXml.startsWith("<html>")) {
                                    console.log(pasteXml);
                                    const addObjectTool = this.interface.getTool("add-object-tool");
                                    addObjectTool.addEntityFromXml(pasteString);
                                }
                            }
                            catch (error) {
                                console.error("Error copy paste adding an object.", error);
                            }
                        });
                        return;
                    }
                }
            }
            // do something with url here
        });
        /*(this.entity.sceneEl!! as any).addEventListener('click', (e: MouseEvent) => {
            console.log('mouse click ' + e.button);
        });

        */
        /*(this.entity.sceneEl!! as any).addEventListener('wheel', (e: WheelEvent) => {
            console.log('wheel: x=' + e.deltaX + ', y=' + e.deltaY + ', z=' + e.deltaZ);

            if (new Date().getTime() - this.lastWheelYTimeMillis > 1000) {

                if (e.deltaY != 0) {
                    if (e.deltaY > 0) {
                        this.interface.buttonDown(this, Slot.PRIMARY, Button.UP);
                        this.interface.buttonUp(this, Slot.PRIMARY, Button.UP);
                    } else {
                        this.interface.buttonDown(this, Slot.PRIMARY, Button.DOWN);
                        this.interface.buttonUp(this, Slot.PRIMARY, Button.DOWN);
                    }

                    this.lastWheelYTimeMillis = new Date().getTime();
                }
            }
        });*/
    }
    update(data, oldData) {
    }
    remove() {
    }
    pause() {
    }
    play() {
    }
    tick(time, timeDelta) {
    }
    onKeyDown(key) {
        if (key == this.movementForwardKey) {
            this.interface.buttonDown(this, Slot_1.Slot.WALK, Button_1.Button.UP);
        }
        if (key == this.movementBackwardKey) {
            this.interface.buttonDown(this, Slot_1.Slot.WALK, Button_1.Button.DOWN);
        }
        if (key == this.movementLeftKey) {
            this.interface.buttonDown(this, Slot_1.Slot.WALK, Button_1.Button.LEFT);
        }
        if (key == this.movementRightKey) {
            this.interface.buttonDown(this, Slot_1.Slot.WALK, Button_1.Button.RIGHT);
        }
        if (key == this.jumpKey) {
            this.interface.buttonDown(this, Slot_1.Slot.WALK, Button_1.Button.TRIGGER);
        }
        if (key == this.upKey || key == 'r') {
            this.interface.buttonDown(this, Slot_1.Slot.PRIMARY, Button_1.Button.UP);
        }
        if (key == this.downKey || key == 'f') {
            this.interface.buttonDown(this, Slot_1.Slot.PRIMARY, Button_1.Button.DOWN);
        }
        if (key == this.rightKey || key == 'e') {
            this.interface.buttonDown(this, Slot_1.Slot.PRIMARY_SELECTOR, Button_1.Button.RIGHT);
        }
        if (key == this.leftKey || key == 'q') {
            this.interface.buttonDown(this, Slot_1.Slot.PRIMARY_SELECTOR, Button_1.Button.LEFT);
        }
    }
    onKeyUp(key) {
        if (key == this.movementForwardKey) {
            this.interface.buttonUp(this, Slot_1.Slot.WALK, Button_1.Button.UP);
        }
        if (key == this.movementBackwardKey) {
            this.interface.buttonUp(this, Slot_1.Slot.WALK, Button_1.Button.DOWN);
        }
        if (key == this.movementLeftKey) {
            this.interface.buttonUp(this, Slot_1.Slot.WALK, Button_1.Button.LEFT);
        }
        if (key == this.movementRightKey) {
            this.interface.buttonUp(this, Slot_1.Slot.WALK, Button_1.Button.RIGHT);
        }
        if (key == this.jumpKey) {
            this.interface.buttonUp(this, Slot_1.Slot.WALK, Button_1.Button.TRIGGER);
        }
        if (key == this.upKey || key == 'r') {
            this.interface.buttonUp(this, Slot_1.Slot.PRIMARY, Button_1.Button.UP);
        }
        if (key == this.downKey || key == 'f') {
            this.interface.buttonUp(this, Slot_1.Slot.PRIMARY, Button_1.Button.DOWN);
        }
        if (key == this.rightKey || key == 'e') {
            this.interface.buttonUp(this, Slot_1.Slot.PRIMARY_SELECTOR, Button_1.Button.RIGHT);
        }
        if (key == this.leftKey || key == 'q') {
            this.interface.buttonUp(this, Slot_1.Slot.PRIMARY_SELECTOR, Button_1.Button.LEFT);
        }
        if (key == 'o') {
            const uiSystem = this.getSystemController("ui");
            uiSystem.pushView("<a-example-view/>");
        }
        if (key == 'i') {
            const uiSystem = this.getSystemController("ui");
            uiSystem.popView();
        }
    }
}
KeyboardAndMouseDevice.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("keyboard-and-mouse", {}, false, false, (component, entity, data) => new KeyboardAndMouseDevice(component, entity, data));
exports.KeyboardAndMouseDevice = KeyboardAndMouseDevice;
//# sourceMappingURL=KeyboardAndMouseDevice.js.map