"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DeviceSlot_1 = require("../model/DeviceSlot");
const Slot_1 = require("../model/Slot");
const Button_1 = require("../model/Button");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
const CodeRealityComponentController_1 = require("../../../component/CodeRealityComponentController");
class PrimaryControllerDevice extends CodeRealityComponentController_1.CodeRealityComponentController {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.controllerName = "";
    }
    init() {
        //console.log(this.componentName + " init");
        this.interface.setDevice(DeviceSlot_1.DeviceSlot.PRIMARY_HAND, this);
        this.addEventListener("controllerconnected", (detail) => {
            this.controllerName = detail.name;
            console.log("controllerconnected " + this.controllerName);
        });
        this.addEventListener("controllerdisconnected", (detail) => {
            console.log("controllerdisconnected " + this.controllerName);
        });
        this.addEventListener("triggerup", (detail) => {
            this.interface.buttonUp(this, Slot_1.Slot.PRIMARY, Button_1.Button.TRIGGER);
            console.log("triggerup " + detail);
        });
        this.addEventListener("triggerdown", (detail) => {
            this.interface.buttonDown(this, Slot_1.Slot.PRIMARY, Button_1.Button.TRIGGER);
            console.log("triggerdown " + detail);
        });
        this.addEventListener("gripup", (detail) => {
            this.interface.buttonUp(this, Slot_1.Slot.PRIMARY, Button_1.Button.GRIP);
            console.log("gripup " + detail);
        });
        this.addEventListener("gripdown", (detail) => {
            this.interface.buttonDown(this, Slot_1.Slot.PRIMARY, Button_1.Button.GRIP);
            console.log("gripdown " + detail);
        });
        this.addEventListener("menuup", (detail) => {
            this.interface.buttonUp(this, Slot_1.Slot.PRIMARY, Button_1.Button.MENU);
            console.log("menuup " + detail);
        });
        this.addEventListener("menudown", (detail) => {
            this.interface.buttonDown(this, Slot_1.Slot.PRIMARY, Button_1.Button.MENU);
            console.log("menudown " + detail);
        });
        this.addEventListener("trackpaddown", (detail) => { this.trackPadDown(detail); });
        this.addEventListener("thumbstickdown", (detail) => { this.trackPadDown(detail); });
        this.addEventListener("trackpadup", (detail) => { this.trackPadUp(detail); });
        this.addEventListener("thumbstickup", (detail) => { this.trackPadUp(detail); });
        this.addEventListener("axismove", (detail) => { this.axisMove(detail); });
    }
    trackPadDown(detail) {
        console.log("trackpaddown " + detail);
        if (this.axis) {
            const button = this.getStickButton(this.axis[0], this.axis[1]);
            if (button == Button_1.Button.UP || button == Button_1.Button.DOWN) {
                this.interface.buttonDown(this, Slot_1.Slot.PRIMARY, button);
            }
            else {
                this.interface.buttonDown(this, Slot_1.Slot.PRIMARY_SELECTOR, button);
            }
            this.lastStickButton = button;
            console.log("button down: " + Button_1.Button[button]);
        }
    }
    trackPadUp(detail) {
        console.log("trackpadup " + detail);
        if (this.axis) {
            if (this.lastStickButton) {
                const button = this.lastStickButton;
                if (button == Button_1.Button.UP || button == Button_1.Button.DOWN) {
                    this.interface.buttonUp(this, Slot_1.Slot.PRIMARY, button);
                }
                else {
                    this.interface.buttonUp(this, Slot_1.Slot.PRIMARY_SELECTOR, button);
                }
                console.log("button up: " + Button_1.Button[button]);
            }
        }
    }
    axisMove(detail) {
        const axis = detail.axis;
        this.axis = Object.assign([], axis);
        if (this.controllerName === "oculus-go-controls") {
            this.axis[1] = -1 * this.axis[1];
        }
        //console.log("axismove " + axis);
        //this.interface.stickTwist(this, Slot.PRIMARY, Stick.PRIMARY, this.axis[1], this.axis[0]);
    }
    getStickButton(x, y) {
        var angle = Math.atan2(y, x) * 180 / Math.PI;
        if (angle >= -45 && angle < 45) {
            return Button_1.Button.RIGHT;
        }
        else if (angle >= 45 && angle < 135) {
            return Button_1.Button.UP;
        }
        else if (angle >= 135 || angle <= -135) {
            return Button_1.Button.LEFT;
        }
        else {
            return Button_1.Button.DOWN;
        }
    }
    update(data, oldData) {
    }
    remove() {
    }
    pause() {
    }
    play() {
    }
    tick(time, timeDelta) {
    }
}
PrimaryControllerDevice.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("primary-controller", {}, false, false, (component, entity, data) => new PrimaryControllerDevice(component, entity, data));
exports.PrimaryControllerDevice = PrimaryControllerDevice;
//# sourceMappingURL=PrimaryControllerDevice.js.map