"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const Slot_1 = require("../model/Slot");
const Button_1 = require("../model/Button");
const PointerTool_1 = require("./PointerTool");
const math_1 = require("../../../math/math");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class AddObjectTool extends PointerTool_1.PointerTool {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.entityTemplateScale = 1;
        this.entityReviewScale = 0.05;
        this.entityTemplates = [
            '<a-entity model="#cube_wood" collidable/>',
            '<a-entity model="#cube_brick" collidable/>',
            '<a-entity model="#cube_grass" collidable/>',
            '<a-entity model="#cube_sand" collidable/>',
            '<a-entity model="#cube_marble" collidable/>',
            '<a-entity model="#cube_stone" collidable/>',
            '<a-entity model="#slab_sand" collidable/>',
            '<a-entity model="#slab_marble" collidable/>',
            '<a-entity model="#slab_stone" collidable/>',
            '<a-entity model="#pedestal" collidable/>'
        ];
        this.entityTemplateIndex = 0;
        this.entityTemplate = this.entityTemplates[this.entityTemplateIndex];
        this.raycaster = new three_1.Raycaster();
    }
    init() {
        //console.log(this.componentName + " init");
        super.init();
    }
    tick(time, timeDelta) {
        super.tick(time, timeDelta);
    }
    buttonDown(device, toolSlot, button) {
        if (!this.pressed.has(button)) {
        }
        super.buttonDown(device, toolSlot, button);
    }
    buttonUp(device, toolSlot, button) {
        if (this.pressed.has(button)) {
            if (button == Button_1.Button.TRIGGER) {
                this.addEntityFromTemplate();
            }
            if (button == Button_1.Button.UP) {
                if (this.pressed.has(Button_1.Button.TRIGGER)) {
                    this.cloneEntity();
                }
                else {
                    this.entityTemplateIndex++;
                    if (this.entityTemplateIndex >= this.entityTemplates.length) {
                        this.entityTemplateIndex = 0;
                    }
                    this.entityTemplate = this.entityTemplates[this.entityTemplateIndex];
                    this.setReviewEntity();
                }
            }
            if (button == Button_1.Button.DOWN) {
                this.entityTemplateIndex--;
                if (this.entityTemplateIndex <= 0) {
                    this.entityTemplateIndex = this.entityTemplates.length - 1;
                }
                this.entityTemplate = this.entityTemplates[this.entityTemplateIndex];
                this.setReviewEntity();
            }
        }
        super.buttonUp(device, toolSlot, button);
    }
    setReviewEntity() {
        const toolSelectorTool = this.interface.getToolAtSlot(Slot_1.Slot.PRIMARY_SELECTOR);
        if (this.reviewEntity) {
            toolSelectorTool.entity.removeChild(this.reviewEntity);
        }
        this.reviewEntity = aframe_typescript_boilerplate_1.createElement(this.entityTemplate);
        this.reviewEntity.setAttribute("scale", this.entityReviewScale + " " + this.entityReviewScale + " " + this.entityReviewScale);
        this.reviewEntity.setAttribute("rotation", "60 0 0");
        this.reviewEntity.setAttribute("position", "0 0 -0.1");
        toolSelectorTool.entity.appendChild(this.reviewEntity);
    }
    cloneEntity() {
        //console.log("clone entity.");
        const gridStep = 1;
        const pointedObject = this.pointedObject;
        const pointerPosition = this.pointedPosition;
        const pointedFace = this.pointedFace;
        if (pointedObject && pointerPosition && pointedFace) {
            const entity = aframe_typescript_boilerplate_1.getEntity(pointedObject);
            const entityXml = entity.outerHTML;
            const currentOrientation = entity.object3D.quaternion.clone();
            const currentScale = entity.object3D.scale;
            const tempEntity = aframe_typescript_boilerplate_1.createElement(entityXml);
            tempEntity.setAttribute("quaternion", currentOrientation.x + " " + currentOrientation.y + " " + currentOrientation.z + " " + currentOrientation.w);
            this.recursiveClearSidAndServer(tempEntity);
            //const pointedObjectPosition = pointedObject.position.clone();
            //pointedObject.getWorldPosition(pointedObjectPosition);
            const template = tempEntity.outerHTML;
            const templateScale = pointedObject.scale.x;
            const entityPosition = pointedObject.localToWorld(pointedObject.position.clone());
            const normal = pointedObject.localToWorld(pointedFace.normal.clone()).sub(pointedObject.localToWorld(pointedObject.position.clone()));
            const newPosition = entityPosition.add(normal.multiplyScalar(templateScale));
            const spaceSystem = this.getSystemController("space");
            spaceSystem.addEntity(template, newPosition, currentScale);
        }
    }
    addEntityFromXml(xml) {
        const gridStep = 1;
        const pointedObject = this.pointedObject;
        const pointerPosition = this.pointedPosition;
        const pointedFace = this.pointedFace;
        if (pointedObject && pointerPosition && pointedFace) {
            const entityXml = xml;
            const tempEntity = aframe_typescript_boilerplate_1.createElement(entityXml);
            this.recursiveClearSidAndServer(tempEntity);
            const template = tempEntity.outerHTML;
            const templateScale = this.interface.getSelfScale() * this.entityTemplateScale;
            const entityPosition = pointerPosition.clone();
            const normal = pointedObject.localToWorld(pointedFace.normal.clone()).sub(pointedObject.localToWorld(pointedObject.position.clone()));
            const newPosition = math_1.snapVector3ToAxisAlignedGrid(entityPosition.add(normal.multiplyScalar(this.interface.getSelfScale() * gridStep / 2)), this.interface.getSelfScale() * gridStep);
            const spaceSystem = this.getSystemController("space");
            spaceSystem.addEntity(template, newPosition, new three_1.Vector3(templateScale, templateScale, templateScale));
        }
    }
    addEntityFromTemplate() {
        const gridStep = 1;
        const pointedObject = this.pointedObject;
        const pointerPosition = this.pointedPosition;
        const pointedFace = this.pointedFace;
        if (pointedObject && pointerPosition && pointedFace) {
            //const pointedObjectPosition = pointedObject.position.clone();
            //pointedObject.getWorldPosition(pointedObjectPosition);
            const template = this.entityTemplate;
            const templateScale = this.interface.getSelfScale() * this.entityTemplateScale;
            const entityPosition = pointerPosition.clone();
            const normal = pointedObject.localToWorld(pointedFace.normal.clone()).sub(pointedObject.localToWorld(pointedObject.position.clone()));
            const newPosition = math_1.snapVector3ToAxisAlignedGrid(entityPosition.add(normal.multiplyScalar(this.interface.getSelfScale() * gridStep / 2)), this.interface.getSelfScale() * gridStep);
            const spaceSystem = this.getSystemController("space");
            spaceSystem.addEntity(template, newPosition, new three_1.Vector3(templateScale, templateScale, templateScale));
        }
    }
    recursiveClearSidAndServer(entity) {
        entity.removeAttribute("sid");
        entity.removeAttribute("server");
        if (entity.children && entity.children.length > 0) {
            for (const child of entity.children) {
                this.recursiveClearSidAndServer(child);
            }
        }
    }
}
AddObjectTool.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("add-object-tool", {}, false, true, (component, entity, data) => new AddObjectTool(component, entity, data));
exports.AddObjectTool = AddObjectTool;
//# sourceMappingURL=AddObjectTool.js.map