"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const Button_1 = require("../model/Button");
const PointerTool_1 = require("./PointerTool");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class MoveObjectTool extends PointerTool_1.PointerTool {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.moveStep = 0.25;
        this.raycaster = new three_1.Raycaster();
    }
    init() {
        super.init();
    }
    tick(time, timeDelta) {
        super.tick(time, timeDelta);
    }
    buttonDown(device, toolSlot, button) {
        if (!this.pressed.has(button)) {
            if (button == Button_1.Button.UP) {
                if (this.pressed.has(Button_1.Button.TRIGGER)) {
                    this.moveForward();
                }
            }
        }
        super.buttonDown(device, toolSlot, button);
    }
    buttonUp(device, toolSlot, button) {
        if (this.pressed.has(button)) {
            if (button == Button_1.Button.DOWN) {
                if (this.pressed.has(Button_1.Button.TRIGGER)) {
                    this.moveBackward();
                }
            }
        }
        super.buttonUp(device, toolSlot, button);
    }
    moveForward() {
        const object = this.pointedObject;
        if (object) {
            const entity = aframe_typescript_boilerplate_1.getEntity(object);
            this.move(entity, this.moveStep);
        }
    }
    moveBackward() {
        const object = this.pointedObject;
        if (object) {
            const entity = aframe_typescript_boilerplate_1.getEntity(object);
            this.move(entity, -this.moveStep);
        }
    }
    move(entity, moveStep) {
        const spaceSystem = this.getSystemController("space");
        const gridStep = 1;
        const pointedObject = this.pointedObject;
        const pointerPosition = this.pointedPosition;
        const pointedFace = this.pointedFace;
        if (pointedObject && pointerPosition && pointedFace) {
            const currentPosition = entity.object3D.getWorldPosition(entity.object3D.position.clone());
            const currentOrientation = entity.object3D.quaternion.clone();
            const normal = pointedObject.localToWorld(pointedFace.normal.clone()).sub(pointedObject.localToWorld(pointedObject.position.clone()));
            const newPosition = currentPosition.add(normal.multiplyScalar(-this.interface.getSelfScale() * gridStep * moveStep));
            const scale = entity.getAttribute("scale");
            spaceSystem.updateEntity(entity, newPosition, currentOrientation, scale);
        }
    }
}
MoveObjectTool.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("move-object-tool", {}, false, true, (component, entity, data) => new MoveObjectTool(component, entity, data));
exports.MoveObjectTool = MoveObjectTool;
//# sourceMappingURL=MoveObjectTool.js.map