"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const Button_1 = require("../model/Button");
const raycast_1 = require("../../../three/raycast");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
const CodeRealityComponentController_1 = require("../../../component/CodeRealityComponentController");
class PointerTool extends CodeRealityComponentController_1.CodeRealityComponentController {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.pointerMaterial = new three_1.LineBasicMaterial({ color: 0xffffff, transparent: true, opacity: 0.5 });
        this.pressed = new Set();
        this.time = 0;
        this.pointerLine = this.constructPointerLine();
        this.pointerCursor = this.constructPointerCursor();
        this.pointerDevicePosition = new three_1.Vector3(0, 0, 0);
        this.pointerDirection = new three_1.Vector3(0, 0, 0);
        this.raycaster = new three_1.Raycaster();
        this.interface.registerTool(this);
    }
    init() {
        //console.log(this.componentName + " init");
    }
    update(data, oldData) { }
    remove() { }
    pause() { }
    play() { }
    tick(time, timeDelta) {
        if (this.pointerDevice) {
            this.pointerTick(time, timeDelta);
        }
    }
    buttonDown(device, toolSlot, button) {
        if (!this.pressed.has(button)) {
            if (button == Button_1.Button.TRIGGER && !this.pointerDevice) {
                this.pointerOn(device);
            }
            this.pressed.add(button);
        }
    }
    buttonUp(device, toolSlot, button) {
        if (this.pressed.has(button)) {
            if (button == Button_1.Button.TRIGGER && this.pointerDevice) {
                this.pointerOff(device);
            }
            this.pressed.delete(button);
        }
    }
    stickTwist(device, toolSlot, stick, x, y) {
    }
    constructPointerLine() {
        var geometry = new three_1.Geometry();
        geometry.vertices.push(new three_1.Vector3(0, 0, -100), new three_1.Vector3(0, 0, 0));
        return new three_1.Line(geometry, this.pointerMaterial);
    }
    constructPointerCursor() {
        return new three_1.Mesh(new three_1.SphereGeometry(0.01, 32, 32), this.pointerMaterial);
    }
    pointerOn(device) {
        device.entity.object3D.add(this.pointerLine);
        this.pointerDevice = device;
    }
    pointerOff(device) {
        if (this.pointedPosition) {
            this.removeCursor();
        }
        device.entity.object3D.remove(this.pointerLine);
        this.pointerDevice = undefined;
    }
    pointerTick(time, timeDelta) {
        this.pointerDevice.entity.object3D.getWorldDirection(this.pointerDirection);
        this.pointerDirection.multiplyScalar(-1);
        this.pointerDevice.entity.object3D.getWorldPosition(this.pointerDevicePosition);
        this.raycaster.near = 0;
        this.raycaster.far = 100;
        this.raycaster.set(this.pointerDevicePosition, this.pointerDirection);
        //const intersects = this.raycaster!!.intersectObjects(this.interface.getCollidables(), true);
        const intersects = raycast_1.raycast(this.interface.getCollidables(), this.raycaster);
        if (intersects.length > 0) {
            if (intersects[0].object === this.pointerCursor) {
                intersects.splice(0, 1);
            }
        }
        if (intersects.length > 0) {
            const intersectionPoint = intersects[0].point;
            //console.log(JSON.stringify(intersects[0]));
            this.pointerCursor.position.copy(intersectionPoint);
            if (!this.pointedPosition) {
                this.addCursor(intersects[0].object);
            }
            this.pointedPosition = intersectionPoint;
            this.pointedObject = intersects[0].object;
            this.pointedFace = intersects[0].face;
        }
        else {
            if (this.pointedPosition) {
                this.removeCursor();
            }
        }
    }
    addCursor(object) {
        this.pointerCursor.scale.x = this.interface.getSelfScale();
        this.pointerCursor.scale.y = this.interface.getSelfScale();
        this.pointerCursor.scale.z = this.interface.getSelfScale();
        this.scene.object3D.add(this.pointerCursor);
        //console.log("add pointer cursor.");
    }
    removeCursor() {
        this.scene.object3D.remove(this.pointerCursor);
        //console.log("remove pointer cursor.");
        this.pointedObject = undefined;
        this.pointedPosition = undefined;
        this.pointedFace = undefined;
    }
}
PointerTool.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("pointer-tool", {}, false, true, (component, entity, data) => new PointerTool(component, entity, data));
exports.PointerTool = PointerTool;
//# sourceMappingURL=PointerTool.js.map