"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const Button_1 = require("../model/Button");
const PointerTool_1 = require("./PointerTool");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class RotateObjectTool extends PointerTool_1.PointerTool {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.rotateDegrees = 22.5;
        this.raycaster = new three_1.Raycaster();
    }
    init() {
        super.init();
    }
    tick(time, timeDelta) {
        super.tick(time, timeDelta);
    }
    buttonDown(device, toolSlot, button) {
        if (!this.pressed.has(button)) {
        }
        super.buttonDown(device, toolSlot, button);
    }
    buttonUp(device, toolSlot, button) {
        if (this.pressed.has(button)) {
            if (button == Button_1.Button.UP) {
                if (this.pressed.has(Button_1.Button.TRIGGER)) {
                    this.rotateForward();
                }
            }
            if (button == Button_1.Button.DOWN) {
                if (this.pressed.has(Button_1.Button.TRIGGER)) {
                    this.resetRotation();
                }
            }
        }
        super.buttonUp(device, toolSlot, button);
    }
    rotateForward() {
        const object = this.pointedObject;
        if (object) {
            const entity = aframe_typescript_boilerplate_1.getEntity(object);
            this.rotate(entity, this.rotateDegrees);
        }
    }
    rotateBackward() {
        const object = this.pointedObject;
        if (object) {
            const entity = aframe_typescript_boilerplate_1.getEntity(object);
            this.rotate(entity, -this.rotateDegrees);
        }
    }
    rotate(entity, rotateDegrees) {
        const spaceSystem = this.getSystemController("space");
        const gridStep = 1;
        const pointedObject = this.pointedObject;
        const pointerPosition = this.pointedPosition;
        const pointedFace = this.pointedFace;
        if (pointedObject && pointerPosition && pointedFace) {
            const normal = pointedObject.localToWorld(pointedFace.normal.clone()).sub(pointedObject.localToWorld(pointedObject.position.clone()));
            const axis = normal;
            const angle = rotateDegrees * (2 * Math.PI) / 360;
            const quaternionRotation = new three_1.Quaternion();
            quaternionRotation.setFromAxisAngle(axis, angle);
            const currentPosition = entity.object3D.getWorldPosition(entity.object3D.position.clone());
            const currentScale = entity.getAttribute("scale");
            const currentOrientation = entity.object3D.quaternion.clone();
            const newOrientation = quaternionRotation.multiply(currentOrientation);
            spaceSystem.updateEntity(entity, currentPosition, newOrientation.normalize(), currentScale);
        }
    }
    resetRotation() {
        const object = this.pointedObject;
        if (object) {
            const entity = aframe_typescript_boilerplate_1.getEntity(object);
            const spaceSystem = this.getSystemController("space");
            const currentPosition = entity.object3D.getWorldPosition(entity.object3D.position.clone());
            const currentScale = entity.getAttribute("scale");
            const newOrientation = new three_1.Quaternion(0, 0, 0, 1);
            spaceSystem.updateEntity(entity, currentPosition, newOrientation, currentScale);
        }
    }
}
RotateObjectTool.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("rotate-object-tool", {}, false, true, (component, entity, data) => new RotateObjectTool(component, entity, data));
exports.RotateObjectTool = RotateObjectTool;
//# sourceMappingURL=RotateObjectTool.js.map