"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const Button_1 = require("../model/Button");
const PointerTool_1 = require("./PointerTool");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class ScaleObjectTool extends PointerTool_1.PointerTool {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.scaleMultiplier = 2;
        this.raycaster = new three_1.Raycaster();
    }
    init() {
        super.init();
    }
    tick(time, timeDelta) {
        super.tick(time, timeDelta);
    }
    buttonDown(device, toolSlot, button) {
        if (!this.pressed.has(button)) {
            if (button == Button_1.Button.UP) {
                if (this.pressed.has(Button_1.Button.TRIGGER)) {
                    this.scaleEntityUp();
                }
                else {
                    this.scaleSelfUp();
                }
            }
        }
        super.buttonDown(device, toolSlot, button);
    }
    buttonUp(device, toolSlot, button) {
        if (this.pressed.has(button)) {
            if (button == Button_1.Button.DOWN) {
                if (this.pressed.has(Button_1.Button.TRIGGER)) {
                    this.scaleEntityDown();
                }
                else {
                    this.scaleSelfDown();
                }
            }
        }
        super.buttonUp(device, toolSlot, button);
    }
    scaleSelfUp() {
        this.scaleSelf(this.scaleMultiplier);
    }
    scaleSelfDown() {
        this.scaleSelf(1 / this.scaleMultiplier);
    }
    scaleSelf(multiplier) {
        this.interface.setSelfScale(this.interface.getSelfScale() * multiplier);
    }
    scaleEntityUp() {
        const object = this.pointedObject;
        if (object) {
            const entity = aframe_typescript_boilerplate_1.getEntity(object);
            this.scaleEntity(entity, this.scaleMultiplier);
        }
    }
    scaleEntityDown() {
        const object = this.pointedObject;
        if (object) {
            const entity = aframe_typescript_boilerplate_1.getEntity(object);
            this.scaleEntity(entity, 1 / this.scaleMultiplier);
        }
    }
    scaleEntity(entity, multiplier) {
        const spaceSystem = this.getSystemController("space");
        const position = entity.object3D.getWorldPosition(entity.object3D.position.clone());
        const currentOrientation = entity.object3D.quaternion.clone();
        const scale = entity.getAttribute("scale").clone();
        scale.multiplyScalar(multiplier);
        spaceSystem.updateEntity(entity, position, currentOrientation, scale);
    }
}
ScaleObjectTool.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("scale-object-tool", {}, false, true, (component, entity, data) => new ScaleObjectTool(component, entity, data));
exports.ScaleObjectTool = ScaleObjectTool;
//# sourceMappingURL=ScaleObjectTool.js.map