"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Button_1 = require("../model/Button");
const Slot_1 = require("../model/Slot");
const EntityActionEventDetail_1 = require("../../../model/EntityActionEventDetail");
const Events_1 = require("../../../model/Events");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
const CodeRealityComponentController_1 = require("../../../component/CodeRealityComponentController");
class ToolSelectorTool extends CodeRealityComponentController_1.CodeRealityComponentController {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.toolSymbolEntities = new Map();
        this.vrMode = false;
        //console.log(this.componentName + " init: " + JSON.stringify(this.data));
        this.interface.slotTool(Slot_1.Slot.PRIMARY_SELECTOR, this);
    }
    init() {
        this.interface.registerSlotListener(Slot_1.Slot.PRIMARY, this);
        aframe_typescript_boilerplate_1.addEntityEventListener(this.scene, "enter-vr", (detail) => {
            this.vrMode = true;
            const toolSelectorObject = this.entity.object3D;
            const vrModePlaceholderObject = document.getElementById("tool-selector-vr-placeholder");
            const desktopModePlaceholderObject = document.getElementById("tool-selector-desktop-placeholder");
            if (vrModePlaceholderObject && desktopModePlaceholderObject) {
                desktopModePlaceholderObject.object3D.remove(toolSelectorObject);
                vrModePlaceholderObject.object3D.add(toolSelectorObject);
            }
            else {
                console.warn("tool-selector placeholders not found.");
            }
        });
        aframe_typescript_boilerplate_1.addEntityEventListener(this.scene, "exit-vr", (detail) => {
            this.vrMode = false;
            const toolSelectorObject = this.entity.object3D;
            const vrModePlaceholderObject = document.getElementById("tool-selector-vr-placeholder");
            const desktopModePlaceholderObject = document.getElementById("tool-selector-desktop-placeholder");
            if (vrModePlaceholderObject && desktopModePlaceholderObject) {
                vrModePlaceholderObject.object3D.remove(toolSelectorObject);
                desktopModePlaceholderObject.object3D.add(toolSelectorObject);
            }
            else {
                console.warn("tool-selector placeholders not found.");
            }
        });
    }
    update(data, oldData) {
        //console.log(this.componentName + " update: " + JSON.stringify(this.data));
    }
    remove() {
        //console.log(this.componentName + " remove");
    }
    pause() {
        //console.log(this.componentName + " pause");
    }
    play() {
        //console.log(this.componentName + " play");
    }
    tick(time, timeDelta) {
    }
    buttonDown(device, toolSlot, button) {
    }
    buttonUp(device, toolSlot, button) {
        if (button === Button_1.Button.LEFT) {
            this.interface.slotPreviousTool(Slot_1.Slot.PRIMARY);
        }
        if (button === Button_1.Button.RIGHT) {
            this.interface.slotNextTool(Slot_1.Slot.PRIMARY);
        }
    }
    stickTwist(device, toolSlot, stick, x, y) {
    }
    onToolSlotted(slot, toolName) {
        if (!this.toolSymbolEntities.has(toolName)) {
            const toolSymbolEntity = aframe_typescript_boilerplate_1.createElement('<a-entity gltf-model="#' + toolName + '" visible="false"></a-entity>');
            this.entity.appendChild(toolSymbolEntity);
            this.toolSymbolEntities.set(toolName, toolSymbolEntity);
        }
        if (this.currentToolName) {
            this.toolSymbolEntities.get(this.currentToolName).setAttribute("visible", false);
        }
        this.toolSymbolEntities.get(toolName).setAttribute("visible", true);
        this.currentToolName = toolName;
        this.interface.interfaceEntity.dispatchEvent(new CustomEvent(Events_1.Events.EVENT_ACTION, { detail: new EntityActionEventDetail_1.EntityActionEventDetail("change-tool", toolName) }));
    }
}
ToolSelectorTool.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("tool-selector-tool", {}, false, false, (component, entity, data) => new ToolSelectorTool(component, entity, data));
exports.ToolSelectorTool = ToolSelectorTool;
//# sourceMappingURL=ToolSelectorTool.js.map