"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const EntityStateEventDetail_1 = require("../../../model/EntityStateEventDetail");
const Events_1 = require("../../../model/Events");
const Slot_1 = require("../model/Slot");
const Button_1 = require("../model/Button");
const Stick_1 = require("../model/Stick");
const raycast_1 = require("../../../three/raycast");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
const CodeRealityComponentController_1 = require("../../../component/CodeRealityComponentController");
class WalkTool extends CodeRealityComponentController_1.CodeRealityComponentController {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.movementSpeed = 0;
        this.rotationSpeed = 0;
        this.height = 0;
        this.width = 0;
        this.jumpStartSpeed = 0;
        this.minY = 0;
        this.forwardKey = Button_1.Button.UP;
        this.backwardKey = Button_1.Button.DOWN;
        this.leftKey = Button_1.Button.LEFT;
        this.rightKey = Button_1.Button.RIGHT;
        this.jumpKey = Button_1.Button.TRIGGER;
        this.yAxisPositive = new three_1.Vector3(0, 1, 0);
        this.yAxisNegative = new three_1.Vector3(0, -1, 0);
        this.xzPlane = new three_1.Plane(this.yAxisPositive);
        this.jumping = false;
        this.airborne = false;
        this.time = 0;
        this.yVelocity = 0;
        this.pressed = new Map();
        this.centerOfMassPosition = new three_1.Vector3(0, 0, 0);
        this.highCenterOfMassPosition = new three_1.Vector3(0, 0, 0);
        this.lowCenterOfMassPosition = new three_1.Vector3(0, 0, 0);
        this.forwardPositionStep = new three_1.Vector3(0, 0, 0);
        this.rightPositionStep = new three_1.Vector3(0, 0, 0);
        this.forwardPosition = new three_1.Vector3(0, 0, 0);
        this.backwardPosition = new three_1.Vector3(0, 0, 0);
        this.rightPosition = new three_1.Vector3(0, 0, 0);
        this.leftPosition = new three_1.Vector3(0, 0, 0);
        this.stickTranslation = new three_1.Vector3(0, 0, 0);
        this.stickRotation = new three_1.Vector3(0, 0, 0);
        this.cameraDirection = new three_1.Vector3(0, 0, 0);
        this.xzCameraDirection = new three_1.Vector3(0, 0, 0);
        this.xDirection = new three_1.Vector3(0, 0, 0);
        this.zDirection = new three_1.Vector3(0, 0, 0);
        this.direction = new three_1.Vector3(0, 0, 0);
        this.interface.registerTool(this);
        this.interface.slotTool(Slot_1.Slot.WALK, this);
    }
    init() {
        //console.log(this.componentName + " init");
        // Configuration
        this.movementSpeed = this.data.movementSpeed;
        this.rotationSpeed = this.data.rotationSpeed;
        this.minY = this.data.minY;
        this.height = this.data.height;
        this.width = this.data.width;
        this.jumpStartSpeed = this.data.jumpStartSpeed;
        // Utility objects
        this.raycaster = new three_1.Raycaster();
        // Constants
        this.yAxisPositive = new three_1.Vector3(0, 1, 0);
        this.yAxisNegative = new three_1.Vector3(0, -1, 0);
        this.xzPlane = new three_1.Plane(this.yAxisPositive);
        // State booleans
        this.jumping = false;
        this.airborne = false;
        // State variables
        this.time = 0;
        this.yVelocity = 0;
        this.cameraDirection = new three_1.Vector3(0, 0, 0);
        this.xzCameraDirection = new three_1.Vector3(0, 0, 0);
        this.xDirection = new three_1.Vector3(0, 0, 0);
        this.zDirection = new three_1.Vector3(0, 0, 0);
    }
    setJumping(state) {
        if (this.jumping !== state) {
            this.jumping = state;
            this.entityStateChange("jumping", this.jumping);
        }
    }
    setAirborne(state) {
        if (this.airborne !== state) {
            this.airborne = state;
            this.entityStateChange("airborne", this.airborne);
        }
    }
    entityStateChange(state, enabled) {
        if (enabled) {
            this.interface.interfaceEntity.dispatchEvent(new CustomEvent(Events_1.Events.EVENT_STATE_BEGIN, { detail: new EntityStateEventDetail_1.EntityStateEventDetail(state) }));
        }
        else {
            this.interface.interfaceEntity.dispatchEvent(new CustomEvent(Events_1.Events.EVENT_STATE_END, { detail: new EntityStateEventDetail_1.EntityStateEventDetail(state) }));
        }
    }
    update(data, oldData) {
    }
    remove() {
    }
    pause() {
    }
    play() {
        // Reused vector variables.
        this.setCenterOfMassFromInterfaceEntity();
    }
    setCenterOfMassFromInterfaceEntity() {
        this.centerOfMassPosition = new three_1.Vector3(0, 0, 0); // Center of mass for collision checks
        this.centerOfMassPosition.x = this.interface.interfaceEntity.object3D.position.x;
        this.centerOfMassPosition.y = this.interface.interfaceEntity.object3D.position.y + this.interface.getSelfScale() * this.height / 2;
        this.centerOfMassPosition.z = this.interface.interfaceEntity.object3D.position.z;
    }
    tick(time, timeDelta) {
        this.time = time;
        if (timeDelta > 40) {
            timeDelta = 40;
        }
        let collidables = this.interface.getCollidables();
        let position = this.interface.interfaceEntity.object3D.position;
        const x = this.centerOfMassPosition.x;
        const z = this.centerOfMassPosition.z;
        this.updateXZ(timeDelta, collidables);
        const blocked = this.updateY(timeDelta, collidables);
        if (blocked) {
            this.centerOfMassPosition.x = x;
            this.centerOfMassPosition.z = z;
        }
        else {
            position.x = this.centerOfMassPosition.x;
            position.z = this.centerOfMassPosition.z;
        }
        if (this.stickRotation.x != 0 || this.stickRotation.y != 0 || this.stickRotation.z != 0) {
            let delta = this.rotationSpeed * timeDelta / 1000.0;
            this.interface.interfaceEntity.object3D.rotation.y -= this.stickRotation.y * delta;
        }
    }
    buttonDown(device, toolSlot, button) {
        if (!this.pressed.has(button)) {
            if (button == this.backwardKey) {
                this.entityStateChange("backward", true);
                this.stickTranslation.x = -1;
            }
            if (button == this.forwardKey) {
                this.entityStateChange("forward", true);
                this.stickTranslation.x = 1;
            }
            if (button == this.leftKey) {
                this.entityStateChange("left", true);
                this.stickTranslation.z = -1;
            }
            if (button == this.rightKey) {
                this.entityStateChange("right", true);
                this.stickTranslation.z = 1;
            }
        }
        this.pressed.set(button, this.time);
    }
    buttonUp(device, toolSlot, button) {
        if (this.pressed.has(button)) {
            if (button == this.backwardKey) {
                this.entityStateChange("backward", false);
                this.stickTranslation.x = 0;
            }
            if (button == this.forwardKey) {
                this.entityStateChange("forward", false);
                this.stickTranslation.x = 0;
            }
            if (button == this.leftKey) {
                this.entityStateChange("left", false);
                this.stickTranslation.z = 0;
            }
            if (button == this.rightKey) {
                this.entityStateChange("right", false);
                this.stickTranslation.z = 0;
            }
            this.pressed.delete(button);
        }
    }
    stickTwist(device, toolSlot, stick, x, y) {
        if (stick == Stick_1.Stick.PRIMARY) {
            this.stickTranslation.x = 1.5 * x;
            this.stickTranslation.z = 1.5 * y;
        }
        if (stick == Stick_1.Stick.SECONDARY) {
            this.stickRotation.x = 1.0 * x;
            this.stickRotation.y = 1.0 * y;
        }
    }
    updateXZ(timeDelta, collidables) {
        this.computeXZDirectionFromCamera();
        if (this.stickTranslation.x != 0 || this.stickTranslation.z != 0) {
            let delta = this.interface.getSelfScale() * this.movementSpeed * timeDelta / 1000.0;
            this.xDirection.copy(this.xzCameraDirection);
            this.zDirection.copy(this.xzCameraDirection);
            this.zDirection.cross(this.yAxisPositive);
            this.xDirection.multiplyScalar(this.stickTranslation.x * delta);
            this.zDirection.multiplyScalar(this.stickTranslation.z * delta);
            this.direction.copy(this.xDirection);
            this.direction.add(this.zDirection);
            if (!this.testCollision(this.direction, collidables)) {
                this.centerOfMassPosition.x += this.direction.x;
                this.centerOfMassPosition.z += this.direction.z;
            }
        }
    }
    updateY(timeDelta, collidables) {
        let position = this.interface.interfaceEntity.object3D.position;
        //this.computeXZDirectionFromCamera();
        this.forwardPositionStep.copy(this.xzCameraDirection);
        this.forwardPositionStep.multiplyScalar(0.25 * this.width * this.interface.getSelfScale());
        this.rightPositionStep.copy(this.xzCameraDirection);
        this.rightPositionStep.cross(this.yAxisPositive);
        this.rightPositionStep.multiplyScalar(0.25 * this.width * this.interface.getSelfScale());
        this.forwardPosition.copy(this.centerOfMassPosition).add(this.forwardPositionStep);
        this.backwardPosition.copy(this.centerOfMassPosition).add(this.forwardPositionStep.multiplyScalar(-1));
        this.rightPosition.copy(this.centerOfMassPosition).add(this.rightPositionStep);
        this.leftPosition.copy(this.centerOfMassPosition).add(this.rightPositionStep.multiplyScalar(-1));
        const distanceToNearestBelow = this.findDistanceToNearestFromPositions([this.centerOfMassPosition,
            this.forwardPosition,
            this.backwardPosition,
            this.rightPosition,
            this.leftPosition
        ], this.yAxisNegative, collidables);
        //console.log(distanceToNearestBelow);
        if (this.pressed.has(this.jumpKey) && !this.jumping && !this.airborne) {
            this.setJumping(true);
            this.yVelocity = this.interface.getSelfScale() * this.jumpStartSpeed;
        }
        let freeDropDelta = this.yVelocity * timeDelta / 1000.0;
        let delta;
        if (distanceToNearestBelow && !this.jumping) {
            let distanceFromBottom = distanceToNearestBelow - this.interface.getSelfScale() * this.height / 2;
            if (distanceFromBottom < 0 && -distanceFromBottom > this.height * this.interface.getSelfScale() * 0.25) {
                // Too high obstacle to climb (greater than 0.25 height)
                return true;
            }
            if (Math.abs(freeDropDelta) > Math.abs(distanceFromBottom) || Math.abs(distanceFromBottom) < 0.1 * this.interface.getSelfScale()) {
                delta = -distanceFromBottom;
                this.setAirborne(false);
            }
            else {
                if (distanceFromBottom && distanceFromBottom < 0) {
                    delta = -freeDropDelta;
                }
                else {
                    delta = freeDropDelta;
                }
                this.setAirborne(true);
            }
        }
        else {
            delta = freeDropDelta;
            this.setAirborne(true);
        }
        if (this.airborne) {
            this.yVelocity -= this.interface.getSelfScale() * 9.81 * timeDelta / 1000.0;
        }
        else {
            this.yVelocity = 0;
        }
        if (this.yVelocity < 0) {
            this.setJumping(false);
        }
        this.centerOfMassPosition.y = this.centerOfMassPosition.y + delta;
        if (this.centerOfMassPosition.y - this.interface.getSelfScale() * this.height / 2 < this.minY) {
            this.centerOfMassPosition.y = this.minY + this.interface.getSelfScale() * this.height / 2;
        }
        position.y = this.centerOfMassPosition.y - this.interface.getSelfScale() * this.height / 2;
        return false;
    }
    computeXZDirectionFromCamera() {
        this.interface.cameraEntity.object3D.getWorldDirection(this.cameraDirection);
        this.cameraDirection.multiplyScalar(-1);
        this.xzPlane.projectPoint(this.cameraDirection, this.xzCameraDirection);
        this.xzCameraDirection.normalize();
    }
    findDistanceToNearestFromPositions(positions, rayDirection, objects) {
        const distances = positions.map(position => {
            return this.findDistanceToNearestFromPosition(position, rayDirection, objects);
        }).filter(distance => distance != null);
        distances.sort(function sortNumber(a, b) {
            return a - b;
        });
        if (distances.length > 0) {
            return distances[0];
        }
        else {
            return undefined;
        }
    }
    findDistanceToNearestFromPosition(position, rayDirection, objects) {
        this.raycaster.near = 0;
        this.raycaster.far = this.height * this.interface.getSelfScale();
        this.raycaster.set(position, rayDirection);
        const caster = this.raycaster;
        const intersects = raycast_1.raycast(objects, caster);
        if (intersects.length > 0) {
            return intersects[0].distance;
        }
        else {
            return null;
        }
    }
    testCollision(direction, objects) {
        this.highCenterOfMassPosition.copy(this.centerOfMassPosition);
        this.highCenterOfMassPosition.y += this.height * this.interface.getSelfScale() / 2;
        this.lowCenterOfMassPosition.copy(this.centerOfMassPosition);
        this.lowCenterOfMassPosition.y -= this.height * this.interface.getSelfScale() / 4;
        let distanceToNearestAhead = this.findDistanceToNearestFromPositions([this.centerOfMassPosition,
            this.highCenterOfMassPosition,
            this.centerOfMassPosition,
            this.lowCenterOfMassPosition
        ], direction, objects);
        let collisionAhead = distanceToNearestAhead && distanceToNearestAhead < this.interface.getSelfScale() * this.width / 2;
        return collisionAhead;
    }
}
WalkTool.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("walk-tool", {
    movementSpeed: { type: 'number', default: 2 },
    rotationSpeed: { type: 'number', default: 1 },
    height: { type: 'number', default: 2 },
    width: { type: 'number', default: 0.5 },
    jumpStartSpeed: { type: 'number', default: 5.0 },
    minY: { type: 'number', default: -1 }
}, false, true, (component, entity, data) => new WalkTool(component, entity, data));
exports.WalkTool = WalkTool;
//# sourceMappingURL=WalkTool.js.map