"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class LoaderSystemController extends aframe_typescript_boilerplate_1.AbstractSystemController {
    constructor(system, scene, data) {
        super(system, scene, data);
        this.loadingCounter = 0;
        this.added = false;
        this.loaderGeometry = new three_1.SphereGeometry(1, 36, 18, 0, 2 * Math.PI, 0, Math.PI);
        this.loaderMaterial = new three_1.MeshBasicMaterial({ color: 'white', side: three_1.DoubleSide, depthTest: false, transparent: true });
        this.loaderObject = new three_1.Mesh(this.loaderGeometry, this.loaderMaterial);
        this.sphereGeometry = new three_1.SphereGeometry(0.01, 36, 18, 0, 2 * Math.PI, 0, Math.PI);
        this.sphereMaterial = new three_1.MeshBasicMaterial({ color: 'gray', depthTest: false, transparent: true });
        this.sphereMesh1 = new three_1.Mesh(this.sphereGeometry, this.sphereMaterial);
        this.sphereMesh2 = this.sphereMesh1.clone();
        this.sphereMesh3 = this.sphereMesh1.clone();
        this.progressTime = 0;
        this.sceneEl = scene;
        this.loaderObject.renderOrder = 1;
        this.loaderObject.position.set(0, 0, 0);
        this.sphereMesh1.position.set(-0.05, 0, -0.3);
        this.sphereMesh1.renderOrder = 2;
        this.sphereMesh2.position.set(0, 0, -0.3);
        this.sphereMesh2.renderOrder = 2;
        this.sphereMesh3.position.set(0.05, 0, -0.3);
        this.sphereMesh3.renderOrder = 2;
        this.loaderObject.add(this.sphereMesh1);
        this.loaderObject.add(this.sphereMesh2);
        this.loaderObject.add(this.sphereMesh3);
    }
    init() {
    }
    pause() {
    }
    play() {
    }
    tick(time, timeDelta) {
        this.progressTime += timeDelta < 25 ? timeDelta : 25;
        if (this.loadingCounter > 0) {
            if (!this.added) {
                this.added = true;
                this.sceneEl.camera.add(this.loaderObject);
            }
            this.sphereMesh1.visible = Math.floor(this.progressTime / 150) % 3 == 0;
            this.sphereMesh2.visible = Math.floor(this.progressTime / 150) % 3 == 1;
            this.sphereMesh3.visible = Math.floor(this.progressTime / 150) % 3 == 2;
        }
        else {
            if (this.added) {
                this.added = false;
                this.sceneEl.camera.remove(this.loaderObject);
            }
        }
    }
    increaseLoadingCounter() {
        this.loadingCounter++;
        console.log("loading counter increase: " + this.loadingCounter);
    }
    ;
    decreaseLoadingCounter() {
        if (this.loadingCounter == 0) {
            console.warn("Notify loaded attempting to drop loading counter lower than zero.");
        }
        this.loadingCounter--;
        console.log("loading counter decrease: " + this.loadingCounter);
    }
    ;
}
LoaderSystemController.DEFINITION = new aframe_typescript_boilerplate_1.SystemControllerDefinition("loader-system", {}, (system, scene, data) => new LoaderSystemController(system, scene, data));
exports.LoaderSystemController = LoaderSystemController;
//# sourceMappingURL=LoaderSystemController.js.map