"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const MergeData_1 = require("./MergeData");
const three_1 = require("three");
const merge_util_1 = require("../../three/merge_util");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class MergeSystemController extends aframe_typescript_boilerplate_1.AbstractSystemController {
    constructor(system, scene, data) {
        super(system, scene, data);
        this.merges = new Map();
        this.startTime = 0;
        this.loadingRegistered = false;
        this.merging = false;
        this.niceTimeoutCounter = 0;
    }
    init() {
        //        (this.getSystemController("loader-system") as LoaderSystemController).increaseLoadingCounter();
        this.startTime = new Date().getTime();
    }
    pause() {
    }
    play() {
    }
    tick(time, timeDelta) {
        /*const timeMillis = new Date().getTime();
        for (const merge of this.merges.values()) {
            if (merge.lastMergeTimeMillis != 0 && merge.lastModificationTimeMillis > merge.lastMergeTimeMillis && timeMillis - merge.lastModificationTimeMillis > 60000) {
                this.merge(merge);
            }
        }
        */
        let loading = new Date().getTime() - this.startTime < 3000;
        for (const merge of this.merges.values()) {
            if (this.merging || merge.loadingChildEntities.size > 3 || merge.mergingChildEntities.size > 3) {
                loading = true;
            }
        }
        if (loading) {
            if (!this.loadingRegistered) {
                this.loadingRegistered = true;
                this.getSystemController("loader-system").increaseLoadingCounter();
                console.log("merge system increases loading counter.");
            }
        }
        else {
            if (this.loadingRegistered) {
                this.getSystemController("loader-system").decreaseLoadingCounter();
                this.loadingRegistered = false;
                console.log("merge system decreases loading counter.");
            }
        }
    }
    addMerge(mergeEntity) {
        //console.log("merge added.");
        if (!this.merges.has(mergeEntity)) {
            this.merges.set(mergeEntity, new MergeData_1.MergeData(mergeEntity));
        }
    }
    addLoadingMergeChild(mergeEntity, mergeChildEntity) {
        ////console.log("merge child loading...");
        if (!this.merges.has(mergeEntity)) {
            this.merges.set(mergeEntity, new MergeData_1.MergeData(mergeEntity));
        }
        const merge = this.merges.get(mergeEntity);
        merge.lastModificationTimeMillis = new Date().getTime();
        merge.childEntities.add(mergeChildEntity);
        merge.loadingChildEntities.add(mergeChildEntity);
    }
    setMergeChildLoaded(mergeEntity, mergeChildEntity) {
        ////console.log("merge child loaded.");
        const merge = this.merges.get(mergeEntity);
        if (!merge) {
            return;
        }
        merge.loadingChildEntities.delete(mergeChildEntity);
        merge.mergingChildEntities.add(mergeChildEntity);
        if (merge.loadingChildEntities.size > 0) {
            ////console.log("merge child entities still loading: " + merge.loadingChildEntities.size);
            return;
        }
        //console.log("merge child entities loaded.");
        //if (merge.lastMergeTimeMillis == 0) {
        this.merge(merge);
        //}
    }
    merge(merge) {
        if (this.merging) {
            return;
        }
        this.merging = true;
        //console.log("merging...");
        const startTimeMillis = new Date().getTime();
        // Collect objects to merge.
        this.collectObjectsToMerge(merge).then((objectsToMerge) => {
            merge.mergingChildEntities.clear();
            //console.log("merging object...");
            merge_util_1.mergeObject3Ds(merge.objectMerge, objectsToMerge).then(() => {
                if (!merge.group) {
                    merge.group = merge.objectMerge.group;
                    merge.entity.object3D.add(merge.group);
                }
                merge.lastMergeTimeMillis = new Date().getTime();
                //console.log("merge done: " + (new Date().getTime() - startTimeMillis) + " ms.");
                this.merging = false;
                //                (this.getSystemController("loader-system") as LoaderSystemController).decreaseLoadingCounter();
            });
        });
    }
    collectObjectsToMerge(merge) {
        return __awaiter(this, void 0, void 0, function* () {
            const objectsToMerge = new Array();
            //console.log("collecting objects to merge from child entities: " + merge.mergingChildEntities.size);
            for (const entity of merge.mergingChildEntities) {
                const clone = yield this.cloneObjectToMergeWithWait(entity, merge);
                objectsToMerge.push(clone);
            }
            return objectsToMerge;
        });
    }
    cloneObjectToMergeWithWait(entity, merge) {
        return new Promise((resolve, reject) => {
            const originalObject = entity.object3D;
            this.allocateMergeObjectIndex(merge.objectMerge, originalObject);
            originalObject.visible = false;
            const clone = this.cloneAndSwitchToRegionCoordinates(originalObject, merge);
            this.niceTimeoutCounter++;
            if (this.niceTimeoutCounter % 25 == 0) {
                setTimeout(() => {
                    resolve(clone);
                }, 1);
            }
            else {
                resolve(clone);
            }
        });
    }
    allocateMergeObjectIndex(objectMerge, object) {
        object.userData.mergeObjectIndex = objectMerge.objectOffset;
        objectMerge.objectOffset++;
        if (object.children) {
            for (const child of object.children) {
                this.allocateMergeObjectIndex(objectMerge, child);
            }
        }
    }
    remove(merge) {
        const startTimeMillis = new Date().getTime();
        merge.lastMergeTimeMillis = startTimeMillis;
        //console.log("removing from merge...");
        //console.log("child entities to remove size: " + merge.removingChildEntities.size);
        // Collect objects to merge.
        const objectsToRemove = new Array();
        for (const entity of merge.removingChildEntities) {
            const originalObject = entity.object3D;
            originalObject.visible = true;
            objectsToRemove.push(originalObject);
        }
        merge.removingChildEntities.clear();
        merge_util_1.clearObject3Ds(merge.objectMerge, objectsToRemove).then(() => {
            //console.log("removing from merge done: " + (new Date().getTime() - startTimeMillis) + " ms.");
        });
    }
    update(merge) {
        const startTimeMillis = new Date().getTime();
        merge.lastMergeTimeMillis = startTimeMillis;
        //console.log("updating merge...");
        //console.log("child entities to update size: " + merge.updatingChildEntities.size);
        // Collect objects to merge.
        const objectsToUpdate = new Array();
        for (const entity of merge.updatingChildEntities) {
            const originalObject = entity.object3D;
            const objectToMerge = this.cloneAndSwitchToRegionCoordinates(originalObject, merge);
            objectsToUpdate.push(objectToMerge);
        }
        merge.updatingChildEntities.clear();
        merge_util_1.updateObject3Ds(merge.objectMerge, objectsToUpdate).then(() => {
            //console.log("updating merge done: " + (new Date().getTime() - startTimeMillis) + " ms.");
        });
    }
    cloneAndSwitchToRegionCoordinates(object, merge) {
        // Clone object to merge and setup coordinates.
        //const objectToMerge = originalObject;
        const clone = merge_util_1.cloneObject3D(object);
        // Transfer to world coordinates as clone does not have parent
        // get world position
        let position = new three_1.Vector3();
        object.updateMatrixWorld(true);
        position.setFromMatrixPosition(object.matrixWorld);
        // convert to merge entity world local coordinates
        merge.entity.object3D.updateMatrixWorld(true);
        position = merge.entity.object3D.worldToLocal(position);
        clone.position.x = position.x;
        clone.position.y = position.y;
        clone.position.z = position.z;
        return clone;
    }
    removeMergeChild(mergeEntity, mergeChildEntity) {
        ////console.log("merge child remove.");
        if (this.merges.has(mergeEntity)) {
            const merge = this.merges.get(mergeEntity);
            merge.childEntities.delete(mergeChildEntity);
            merge.loadingChildEntities.delete(mergeChildEntity);
            merge.removingChildEntities.add(mergeChildEntity);
            this.remove(merge);
        }
    }
    updateMergeChild(mergeEntity, mergeChildEntity) {
        ////console.log("merge child remove.");
        if (this.merges.has(mergeEntity)) {
            const merge = this.merges.get(mergeEntity);
            merge.updatingChildEntities.add(mergeChildEntity);
            this.update(merge);
        }
    }
    removeMerge(mergeEntity) {
        //console.log("merge remove.");
        const merge = this.merges.get(mergeEntity);
        if (merge) {
            this.merges.delete(mergeEntity);
            // Remove old merge object.
            if (merge.group) {
                merge.entity.object3D.remove(merge.group);
            }
        }
    }
}
MergeSystemController.DEFINITION = new aframe_typescript_boilerplate_1.SystemControllerDefinition("merge", {}, (system, scene, data) => new MergeSystemController(system, scene, data));
exports.MergeSystemController = MergeSystemController;
//# sourceMappingURL=MergeSystemController.js.map