"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const gltf_load_1 = require("../../three/gltf_load");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class ModelController extends aframe_typescript_boilerplate_1.AbstractComponentController {
    constructor(component, entity, data) {
        super(component, entity, data);
        this.mergeSystem = aframe_typescript_boilerplate_1.getSystemController(this.entity.sceneEl, "merge");
    }
    init() {
        //console.log(this.componentName + " init: " + JSON.stringify(this.data));
        this.merge = this.recursiveFindMergeParent(this.entity);
        if (this.merge) {
            this.mergeSystem.addLoadingMergeChild(this.merge, this.entity);
        }
    }
    update(data, oldData) {
        //console.log(this.componentName + " update: " + JSON.stringify(this.data));
        gltf_load_1.setEntityGltfModel(this.entity, data).then(() => {
            //console.log("Static model loaded: " + data);
            if (this.merge) {
                this.entity.object3D.visible = false;
                this.mergeSystem.setMergeChildLoaded(this.merge, this.entity);
            }
        }).catch((error) => {
            console.error("Error loading static model:" + data, error);
            if (this.merge) {
                this.entity.object3D.visible = false;
                this.mergeSystem.setMergeChildLoaded(this.merge, this.entity);
            }
        });
    }
    remove() {
        if (this.merge) {
            //console.log("Removing merge child due to model removal.");
            this.mergeSystem.removeMergeChild(this.merge, this.entity);
        }
    }
    pause() {
    }
    play() {
    }
    tick(time, timeDelta) {
        console.error("model tick should not happen.");
    }
    recursiveFindMergeParent(entity) {
        const parent = entity.parentElement;
        if (!parent) {
            return undefined;
        }
        if (parent.hasAttribute("merge")) {
            return parent;
        }
        else {
            return this.recursiveFindMergeParent(parent);
        }
    }
}
ModelController.DEFINITION = new aframe_typescript_boilerplate_1.ComponentControllerDefinition("model", { type: 'model' }, false, false, (component, entity, data) => new ModelController(component, entity, data));
exports.ModelController = ModelController;
//# sourceMappingURL=ModelController.js.map