"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const Spring_1 = require("./Spring");
const Events_1 = require("../../model/Events");
const EntityStateEventDetail_1 = require("../../model/EntityStateEventDetail");
const States_1 = require("../../model/States");
const xml_js_1 = require("xml-js");
const EntityActionEventDetail_1 = require("../../model/EntityActionEventDetail");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class Actuator {
    constructor(scene, region, id, index, description) {
        this.springOne = new Spring_1.Spring();
        this.springTwo = new Spring_1.Spring();
        this.lastPosition = new three_1.Vector3();
        this.facingDirection = new three_1.Vector3(0, 0, -1);
        this.targetOrientation = new three_1.Quaternion();
        this.currentOrientation = new three_1.Quaternion();
        this.lastUpdateTime = 0;
        this.averageUpdateInterval = 0.200;
        this.moving = false;
        this.turning = false;
        this.firstSimulate = true;
        this.cameraPosition = new three_1.Vector3(0, 0, 0);
        this.scene = scene;
        this.region = region;
        this.id = id;
        this.index = index;
        this.description = description;
        this.entity = aframe_typescript_boilerplate_1.createElement(description);
        this.entity.setAttribute("did", id);
        this.entity.setAttribute("server", region);
        this.entity.setAttribute("shadow", "");
        this.springOne.relaxationTime = 0.2;
        this.springTwo.relaxationTime = 0.2;
        this.stateSystemController = aframe_typescript_boilerplate_1.getSystemController(this.scene, "state");
        this.movementState = this.stateSystemController.getState(this.entity, States_1.States.STATE_MOVEMENT);
    }
    changeRegion(region, index) {
        this.entity.setAttribute("server", region);
        this.region = region;
        this.index = index;
    }
    added(x, y, z, rx, ry, rz, rw) {
        this.scene.appendChild(this.entity);
        this.springOne.currentPosition.x = x;
        this.springOne.currentPosition.y = y;
        this.springOne.currentPosition.z = z;
        this.springOne.targetPosition.x = x;
        this.springOne.targetPosition.y = y;
        this.springOne.targetPosition.z = z;
        this.springOne.currentOrientation.x = rx;
        this.springOne.currentOrientation.y = ry;
        this.springOne.currentOrientation.z = rz;
        this.springOne.currentOrientation.w = rw;
        this.springOne.targetOrientation.x = rx;
        this.springOne.targetOrientation.y = ry;
        this.springOne.targetOrientation.z = rz;
        this.springOne.targetOrientation.w = rw;
        this.springTwo.currentPosition.x = x;
        this.springTwo.currentPosition.y = y;
        this.springTwo.currentPosition.z = z;
        this.springTwo.targetPosition.x = x;
        this.springTwo.targetPosition.y = y;
        this.springTwo.targetPosition.z = z;
        this.springTwo.currentOrientation.x = rx;
        this.springTwo.currentOrientation.y = ry;
        this.springTwo.currentOrientation.z = rz;
        this.springTwo.currentOrientation.w = rw;
        this.springTwo.targetOrientation.x = rx;
        this.springTwo.targetOrientation.y = ry;
        this.springTwo.targetOrientation.z = rz;
        this.springTwo.targetOrientation.w = rw;
        this.lastPosition.x = this.springTwo.currentPosition.x;
        this.lastPosition.y = this.springTwo.currentPosition.y;
        this.lastPosition.z = this.springTwo.currentPosition.z;
        //this.moving = true;
        //this.turning = true;
        //this.entity.dispatchEvent(new CustomEvent(Events.EVENT_STATE_BEGIN, {detail: new EntityStateEventDetail("moving")}));
        //console.log(this.entity.tagName + ":" + "start moving");
        /*this.entity.object3D.position.x = this.springTwo.currentPosition.x;
        this.entity.object3D.position.y = this.springTwo.currentPosition.y;
        this.entity.object3D.position.z = this.springTwo.currentPosition.z;
        this.entity.object3D.rotation.setFromQuaternion(this.springTwo.currentOrientation);

        if (this.entity.tagName != "A-DODECAHEDRON") {
            console.log(this.entity.tagName + ":" + JSON.stringify(this.entity.object3D.position));
        }*/
        //this.moving = true;
        //this.entity.dispatchEvent(new CustomEvent(Events.EVENT_STATE_BEGIN, {detail: new EntityStateEventDetail("moving")}));
        //console.log("start moving");
        //this.simulate(1);
    }
    updated(x, y, z, rx, ry, rz, rw) {
        const time = new Date().getTime() / 1000.0;
        if (this.lastUpdateTime != 0) {
            this.averageUpdateInterval = (this.averageUpdateInterval * 9 + (time - this.lastUpdateTime)) / 10;
            this.springOne.relaxationTime = 2 * this.averageUpdateInterval;
            this.springTwo.relaxationTime = 2 * this.averageUpdateInterval;
        }
        this.lastUpdateTime = time;
        //this.entity.setAttribute("position", x + " " + y + " " + z);
        this.springOne.targetPosition.x = x;
        this.springOne.targetPosition.y = y;
        this.springOne.targetPosition.z = z;
        this.springOne.targetOrientation.x = rx;
        this.springOne.targetOrientation.y = ry;
        this.springOne.targetOrientation.z = rz;
        this.springOne.targetOrientation.w = rw;
        if (!this.moving) {
            this.checkIfMoving();
        }
    }
    removed() {
        if (this.entity.parentElement) {
            this.entity.parentElement.removeChild(this.entity);
        }
    }
    described(description) {
        this.description = description;
        const element = xml_js_1.xml2js(description).elements[0];
        if (element.attributes) {
            // TODO recursive updates.
            for (const attributeName in element.attributes) {
                const attributeValue = element.attributes[attributeName];
                if (attributeName == 'gltf-model') {
                    continue;
                }
                if (attributeName == 'avatar') {
                    continue;
                }
                this.entity.setAttribute(attributeName, attributeValue);
            }
        }
    }
    acted(action, description) {
        if (action === 'state-begin') {
            this.entity.dispatchEvent(new CustomEvent(Events_1.Events.EVENT_STATE_BEGIN, { detail: new EntityStateEventDetail_1.EntityStateEventDetail(description) }));
        }
        else if (action === 'state-end') {
            this.entity.dispatchEvent(new CustomEvent(Events_1.Events.EVENT_STATE_END, { detail: new EntityStateEventDetail_1.EntityStateEventDetail(description) }));
        }
        else {
            this.entity.dispatchEvent(new CustomEvent(Events_1.Events.EVENT_ACTION, { detail: new EntityActionEventDetail_1.EntityActionEventDetail(action, description) }));
        }
    }
    simulate(t) {
        if (this.moving || this.turning || this.firstSimulate) {
            this.firstSimulate = false;
            this.lastPosition.x = this.springTwo.currentPosition.x;
            this.lastPosition.y = this.springTwo.currentPosition.y;
            this.lastPosition.z = this.springTwo.currentPosition.z;
            this.springOne.simulate(t);
            this.springTwo.targetPosition.x = this.springOne.currentPosition.x;
            this.springTwo.targetPosition.y = this.springOne.currentPosition.y;
            this.springTwo.targetPosition.z = this.springOne.currentPosition.z;
            this.springTwo.targetOrientation.x = this.springOne.currentOrientation.x;
            this.springTwo.targetOrientation.y = this.springOne.currentOrientation.y;
            this.springTwo.targetOrientation.z = this.springOne.currentOrientation.z;
            this.springTwo.targetOrientation.w = this.springOne.currentOrientation.w;
            this.springTwo.simulate(t);
            if (this.entity.object3D) {
                // Update location only after 3d presentation is ready.
                this.entity.object3D.position.x = this.springTwo.currentPosition.x;
                this.entity.object3D.position.y = this.springTwo.currentPosition.y;
                this.entity.object3D.position.z = this.springTwo.currentPosition.z;
                /*if (this.entity.tagName != "A-DODECAHEDRON") {
                    console.log(this.entity.tagName + ":" + t + ":" + JSON.stringify(this.entity.object3D.position));
                }*/
                this.entity.object3D.rotation.setFromQuaternion(this.springTwo.currentOrientation);
                this.movementState.distanceDelta = Math.abs(this.lastPosition.distanceTo(this.springTwo.currentPosition));
                this.movementState.timeDeltaSeconds = t;
                const movementDirection = this.lastPosition.sub(this.springTwo.currentPosition);
                this.facingDirection.x = 0;
                this.facingDirection.y = 0;
                this.facingDirection.z = -1;
                this.facingDirection = this.facingDirection.applyQuaternion(this.entity.object3D.quaternion);
                this.movementState.facing = movementDirection.angleTo(this.facingDirection) < Math.PI / 2 ? 1 : -1;
            }
            this.checkIfMoving();
        }
        // Check if dynamic object is in observation range. Do this every round to keep the cost independent of whether user is moving or not.
        /*const spaceSystem = getSystemController(this.scene, "space") as SpaceSystemController;
        const interfaceSystem = getSystemController(this.scene, "interface") as InterfaceSystemController;
        const range = spaceSystem.getObservationRange();
        //console.log("Observation range: "+ range);
        this.cameraPosition.copy(interfaceSystem.cameraPosition);
        this.cameraPosition.sub(this.entity.object3D.position);
        const distanceToCamera = this.cameraPosition.length();
        this.entity.object3D.visible = distanceToCamera < range;*/
    }
    checkIfMoving() {
        const positionDelta = this.springOne.targetPosition.distanceTo(this.springTwo.currentPosition);
        this.targetOrientation.x = this.springOne.targetOrientation.x;
        this.targetOrientation.y = this.springOne.targetOrientation.y;
        this.targetOrientation.z = this.springOne.targetOrientation.z;
        this.targetOrientation.w = this.springOne.targetOrientation.w;
        this.currentOrientation.x = this.springTwo.currentOrientation.x;
        this.currentOrientation.y = this.springTwo.currentOrientation.y;
        this.currentOrientation.z = this.springTwo.currentOrientation.z;
        this.currentOrientation.w = this.springTwo.currentOrientation.w;
        const dot = this.targetOrientation.dot(this.currentOrientation);
        const orientationDelta = Math.acos(2 * dot * dot - 1);
        //console.log(orientationDelta);
        //const orientationDelta = ((this.springOne.targetOrientation) as any).angleTo(this.springTwo.currentOrientation) as number;
        const moving = positionDelta > 0.2;
        const turning = orientationDelta > 0.2;
        //console.log(positionDelta);
        if (!this.moving && moving) {
            this.entity.dispatchEvent(new CustomEvent(Events_1.Events.EVENT_STATE_BEGIN, { detail: new EntityStateEventDetail_1.EntityStateEventDetail("moving") }));
            //console.log(this.entity.tagName + ":" + "start moving");
        }
        if (this.moving && !moving) {
            this.entity.dispatchEvent(new CustomEvent(Events_1.Events.EVENT_STATE_END, { detail: new EntityStateEventDetail_1.EntityStateEventDetail("moving") }));
            //console.log(this.entity.tagName + ":" + "end moving");
        }
        this.moving = moving;
        this.turning = turning;
    }
}
exports.Actuator = Actuator;
//# sourceMappingURL=Actuator.js.map