"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Actuator_1 = require("./Actuator");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class DynamicSpace {
    constructor(scene, avatarId) {
        this.avatarIndex = -1;
        this.actuatorsMap = new Map();
        this.actuatorRemoves = new Map();
        this.avatarId = avatarId;
        this.scene = scene;
    }
    connected(region) {
        this.actuatorsMap.set(region, new Map());
    }
    disconnected(region) {
        if (!this.actuatorsMap.has(region)) {
            return;
        }
        this.actuatorsMap.get(region).forEach((value, key) => {
            value.removed();
        });
        this.actuatorsMap.delete(region);
    }
    added(region, index, id, x, y, z, rx, ry, rz, rw, description) {
        if (id === this.avatarId) {
            this.avatarIndex = index;
            return;
        }
        if (this.actuatorRemoves.has(id)) {
            // In case of fast reconnect remove actuator from removal set and move actuator to new region.
            const actuator = this.actuatorRemoves.get(id);
            if (actuator) {
                this.actuatorRemoves.delete(id);
                // Change region.
                this.actuatorsMap.get(actuator.region).delete(actuator.index);
                this.actuatorsMap.get(region).set(index, actuator);
                actuator.changeRegion(region, index);
                return;
            }
        }
        const actuators = this.actuatorsMap.get(region);
        if (!actuators) {
            console.log("Region does not have actuators map i.e. client not connected: " + region);
            return;
        }
        if (actuators.has(index)) {
            console.log("object already added.");
            return;
        }
        const actuator = new Actuator_1.Actuator(this.scene, region, id, index, description);
        actuators.set(index, actuator);
        actuator.added(x, y, z, rx, ry, rz, rw);
    }
    updated(region, index, x, y, z, rx, ry, rz, rw) {
        if (index === this.avatarIndex) {
            //console.log("dataspace - observed own avatar update.");
            return;
        }
        const actuators = this.actuatorsMap.get(region);
        if (!actuators) {
            return;
        }
        const actuator = actuators.get(index);
        if (!actuator) {
            return;
        }
        actuator.updated(x, y, z, rx, ry, rz, rw);
    }
    removed(region, index, id) {
        if (index == this.avatarIndex) {
            //console.log("dataspace - observed own avatar remove.")
            return;
        }
        const actuators = this.actuatorsMap.get(region);
        if (!actuators) {
            return;
        }
        const actuator = actuators.get(index);
        if (!actuator) {
            return;
        }
        this.actuatorRemoves.set(id, actuator);
        setTimeout(() => {
            if (this.actuatorRemoves.has(id)) {
                this.actuatorRemoves.delete(id);
                const actuators = this.actuatorsMap.get(region);
                if (!actuators) {
                    return;
                }
                const actuator = actuators.get(index);
                if (!actuator) {
                    return;
                }
                actuator.removed();
                aframe_typescript_boilerplate_1.getSystemController(this.scene, "state").removeStates(actuator.entity);
                actuators.delete(index);
            }
        }, 1000);
    }
    described(region, index, description) {
        if (index == this.avatarIndex) {
            return;
        }
        const actuators = this.actuatorsMap.get(region);
        if (!actuators) {
            return;
        }
        const actuator = actuators.get(index);
        if (!actuator) {
            return;
        }
        actuator.described(description);
    }
    acted(region, index, action, description) {
        if (index == this.avatarIndex) {
            return;
        }
        const actuators = this.actuatorsMap.get(region);
        if (!actuators) {
            return;
        }
        const actuator = actuators.get(index);
        if (!actuator) {
            return;
        }
        actuator.acted(action, description);
    }
    simulate(t) {
        this.actuatorsMap.forEach(((actuators) => {
            actuators.forEach((actuator => {
                actuator.simulate(t);
            }));
        }));
    }
}
exports.DynamicSpace = DynamicSpace;
//# sourceMappingURL=DynamicSpace.js.map