"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("uuid");
const three_1 = require("three");
const Events_1 = require("../../model/Events");
const DynamicSpace_1 = require("./DynamicSpace");
const StaticSpace_1 = require("./StaticSpace");
const reality_space_1 = require("reality-space");
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
class SpaceSystemController extends aframe_typescript_boilerplate_1.AbstractSystemController {
    constructor(system, scene, data) {
        super(system, scene, data);
        this.clusterUrl = undefined;
        this.avatarId = uuid.v4();
        this.playerElement = null;
        this.cameraElement = null;
        this.playerObject = undefined;
        this.client = undefined;
        this.dynamicSpace = undefined;
        this.staticSpace = undefined;
        this.lastRefresh = 0;
        this.yAxisPositive = new three_1.Vector3(0, 1, 0);
        this.xzPlane = new three_1.Plane(this.yAxisPositive);
        this.cameraDirection = new three_1.Vector3(0, 0, 0);
        this.xzDirection = new three_1.Vector3(0, 0, 0);
        this.directionMatrix = new three_1.Matrix4();
        this.directionQuaternion = new three_1.Quaternion();
        this.observationRange = 0;
        this.staticSpace = new StaticSpace_1.StaticSpace(this.scene);
        fetch('/api/context', {
            credentials: 'same-origin'
        }).then((response) => {
            response.json().then((responseData) => {
                const context = responseData;
                this.clusterUrl = context.clusterUrl;
                this.space = context.space;
                this.idToken = context.idToken;
                this.context = context;
                this.dynamicSpace = new DynamicSpace_1.DynamicSpace(this.scene, this.context.userId + ":" + this.avatarId);
                //console.log("cluster URL: " + this.clusterUrl);
            });
        }).catch((err) => {
            console.error(err);
        });
        /*fetch('/api/users/current/cluster-url', {
            credentials: 'same-origin'
        })
            .then((response) => {
                response.text().then((responseData) => {
                    this.clusterUrl = responseData;
                    //console.log("cluster URL: " + this.clusterUrl);
                });
            }).catch((err) => {
            console.error(err);
        });
        fetch('/api/users/current/space', {
            credentials: 'same-origin'
        })
            .then((response) => {
                response.text().then((responseData) => {
                    this.space = responseData;
                    //console.log("current space: " + this.space);
                });
            }).catch((err) => {
            console.error(err);
        });
        fetch('/api/users/current/id-token', {
            credentials: 'same-origin'
        })
            .then((response) => {
                response.text().then((responseData) => {
                    this.idToken = responseData;
                });
            }).catch((err) => {
            console.error(err);
        });*/
    }
    init() {
    }
    pause() {
        if (this.client) {
            this.client.close();
            this.client = undefined;
        }
    }
    play() {
    }
    tick(time, timeDelta) {
        if (!this.client) {
            this.setupClient();
        }
        else {
            this.dynamicSpace.simulate(timeDelta / 1000);
            if (time - this.lastRefresh > 150) {
                if (this.playerObject && this.cameraObject) {
                    if (this.client.clusterConfiguration) {
                        this.cameraObject.getWorldDirection(this.cameraDirection);
                        //this.cameraDirection.multiplyScalar(-1);
                        this.xzPlane.projectPoint(this.cameraDirection, this.xzDirection);
                        this.xzDirection.normalize();
                        this.directionMatrix.lookAt(new three_1.Vector3(0, 0, 0), this.xzDirection, this.yAxisPositive);
                        this.directionQuaternion.setFromRotationMatrix(this.directionMatrix);
                        this.client.refresh(this.playerObject.position.x, this.playerObject.position.y, this.playerObject.position.z, this.directionQuaternion.x, this.directionQuaternion.y, this.directionQuaternion.z, this.directionQuaternion.w);
                    }
                }
                this.lastRefresh = time;
            }
        }
    }
    setupClient() {
        if (!this.clusterUrl || !this.idToken || !this.space) {
            return;
        }
        this.playerElement = document.querySelector("[interface]");
        this.cameraElement = this.playerElement.querySelector('[camera]');
        if (!this.playerElement) {
            //console.log("dataspace - did not find player element in dom.");
        }
        else {
            this.playerElement.addEventListener(Events_1.Events.EVENT_STATE_BEGIN, ((e) => __awaiter(this, void 0, void 0, function* () {
                if (this.client && this.client.clusterConfiguration) {
                    ////console.log(e.detail);
                    yield this.client.act(this.avatarId, Events_1.Events.EVENT_STATE_BEGIN, e.detail.state);
                }
            })));
            this.playerElement.addEventListener(Events_1.Events.EVENT_STATE_END, ((e) => __awaiter(this, void 0, void 0, function* () {
                if (this.client && this.client.clusterConfiguration) {
                    ////console.log(e.detail);
                    yield this.client.act(this.avatarId, Events_1.Events.EVENT_STATE_END, e.detail.state);
                }
            })));
            this.playerElement.addEventListener(Events_1.Events.EVENT_ACTION, ((e) => __awaiter(this, void 0, void 0, function* () {
                if (this.client && this.client.clusterConfiguration) {
                    ////console.log(e.detail);
                    const actionEventDetail = e.detail;
                    yield this.client.act(this.avatarId, actionEventDetail.action, actionEventDetail.description);
                }
            })));
        }
        if (!this.cameraElement) {
            //console.log("dataspace - did not find camera element in dom.");
        }
        //console.log(this.systemName + " setting up client...");
        if (this.playerElement && this.cameraElement) {
            if (!this.playerObject || !this.cameraObject) {
                this.playerObject = this.playerElement.object3D;
                if (this.cameraElement) {
                    this.cameraObject = this.cameraElement.object3D;
                }
                if (!this.playerObject) {
                    //console.log("No player object.");
                }
                if (!this.cameraObject) {
                    //console.log("No camera object.");
                }
            }
        }
        if (this.playerObject && this.cameraObject) {
            this.client = new reality_space_1.ClusterClient(this.clusterUrl, this.space, this.avatarId, this.playerObject.position.x, this.playerObject.position.y, this.playerObject.position.z, this.cameraObject.quaternion.x, this.cameraObject.quaternion.y, this.cameraObject.quaternion.z, this.cameraObject.quaternion.w, this.getAvatarDescription(), this.idToken);
            this.client.onReceive = (region, type, message) => {
                ////console.log(message);
                if (type === reality_space_1.Encode.ADDED) {
                    const m = reality_space_1.Decode.added(message);
                    this.dynamicSpace.added(region, m[0], m[1], m[2], m[3], m[4], m[5], m[6], m[7], m[8], m[9]);
                }
                if (type === reality_space_1.Encode.UPDATED) {
                    const m = reality_space_1.Decode.updated(message);
                    this.dynamicSpace.updated(region, m[0], m[1], m[2], m[3], m[4], m[5], m[6], m[7]);
                }
                if (type === reality_space_1.Encode.REMOVED) {
                    const m = reality_space_1.Decode.removed(message);
                    this.dynamicSpace.removed(region, m[0], m[1]);
                }
                if (type === reality_space_1.Encode.DESCRIBED) {
                    const m = reality_space_1.Decode.described(message);
                    this.dynamicSpace.described(region, m[0], m[1]);
                }
                if (type === reality_space_1.Encode.ACTED) {
                    const m = reality_space_1.Decode.acted(message);
                    this.dynamicSpace.acted(region, m[0], m[1], m[2]);
                }
            };
            this.client.onStoredRootEntityReceived = (region, sid, entityXml) => {
                this.staticSpace.setRootEntity(region, sid, entityXml);
            };
            this.client.onStoredChildEntityReceived = (region, parentSid, sid, entityXml) => {
                this.staticSpace.setChildEntity(region, parentSid, sid, entityXml);
            };
            this.client.onStoredEntityRemoved = (region, sid) => {
                this.staticSpace.removeEntity(region, sid);
            };
            this.client.onLoaded = (region) => {
                this.staticSpace.loaded(region);
            };
            this.client.onConnectError = (region, error) => {
                document.location.href = "/connection_error.html";
            };
            this.client.onConnect = (region) => {
                //console.log("dataspace - connected: " + region);
                const regionConfiguration = this.client.getRegionConfiguration(region);
                const regionX = regionConfiguration.x;
                const regionY = regionConfiguration.y;
                const regionZ = regionConfiguration.z - 2; // TODO Remove this test
                this.dynamicSpace.connected(region);
                this.staticSpace.connected(region, new three_1.Vector3(regionX, regionY, regionZ));
            };
            this.client.onDisconnect = (region) => {
                //console.log("dataspace - disconnected: " + region)
                this.dynamicSpace.disconnected(region);
                this.staticSpace.disconnected(region);
            };
            this.client.connect().then(() => {
                if (this.client && this.client.clusterConfiguration) {
                    this.observationRange = Math.trunc(this.client.clusterConfiguration.range / this.client.clusterConfiguration.step) * this.client.clusterConfiguration.step;
                    const interfaceSystem = aframe_typescript_boilerplate_1.getSystemController(this.scene, "interface");
                    const observationRange = this.getObservationRange();
                    console.log("Set camera far to: " + observationRange);
                    interfaceSystem.cameraEntity.object3D.far = observationRange;
                }
            }).catch((error) => {
                console.warn("dataspace - cluster client connect error.", error);
                this.client = undefined;
            });
        }
    }
    getObservationRange() {
        return this.observationRange;
    }
    sendAvatarDescriptionUpdate() {
        if (this.client) {
            const avatarDescription = this.getAvatarDescription();
            //console.log("Describing avatar: " + avatarDescription);
            this.client.describe(this.avatarId, this.getAvatarDescription()).catch((error) => {
                //console.log("Error describing avatar: " + error);
            });
        }
    }
    getAvatarDescription() {
        const interfaceSystem = this.getSystemController("interface");
        const selfScale = interfaceSystem ? interfaceSystem.getSelfScale() : 1;
        const avatarScale = selfScale * 0.4; // TODO fix the model size.
        return '<a-entity gltf-model="#male_suit" scale="' + avatarScale + ' ' + avatarScale + ' ' + avatarScale + '" avatar=""></a-entity>';
    }
    addEntity(template, position, scale) {
        if (this.client) {
            const region = this.client.getRegion(position.x, position.y, position.z);
            const regionConfiguration = this.client.getRegionConfiguration(region);
            const localPosition = position.clone();
            localPosition.sub(new three_1.Vector3(regionConfiguration.x, regionConfiguration.y, regionConfiguration.z));
            localPosition.z = localPosition.z + 2;
            const newEntity = aframe_typescript_boilerplate_1.createElement(template);
            newEntity.setAttribute("scale", scale.x + " " + scale.y + " " + scale.z);
            newEntity.setAttribute("position", localPosition.x + " " + localPosition.y + " " + localPosition.z);
            newEntity.setAttribute("oid", uuid.v4().toString());
            if (newEntity.flushToDOM) {
                newEntity.flushToDOM(true);
            }
            this.staticSpace.regionElements.get(region).appendChild(newEntity);
            this.client.storeEntity(newEntity.outerHTML, position.x, position.y, position.z).catch(error => {
                console.error("Error saving entity", error);
            });
        }
    }
    updateEntity(entity, position, orientation, scale) {
        if (this.client) {
            const sid = entity.getAttribute("sid");
            this.staticSpace.ignoreNextSetSids.push(sid);
            const region = this.client.getRegion(position.x, position.y, position.z);
            const regionConfiguration = this.client.getRegionConfiguration(region);
            const localPosition = position.clone();
            localPosition.sub(new three_1.Vector3(regionConfiguration.x, regionConfiguration.y, regionConfiguration.z));
            localPosition.z = localPosition.z + 2;
            const modelController = aframe_typescript_boilerplate_1.getComponentController(entity, "model");
            if (modelController && modelController.merge) {
                //console.log("updating merge as entity is part of merge.");
                entity.setAttribute("position", localPosition.x + " " + localPosition.y + " " + localPosition.z);
                entity.setAttribute("quaternion", orientation.x + " " + orientation.y + " " + orientation.z + " " + orientation.w);
                entity.setAttribute("scale", scale.x + " " + scale.y + " " + scale.z);
                const mergeSystem = this.getSystemController("merge");
                mergeSystem.updateMergeChild(modelController.merge, entity);
            }
            const entityXml = entity.outerHTML;
            const newEntity = aframe_typescript_boilerplate_1.createElement(entityXml);
            newEntity.setAttribute("position", localPosition.x + " " + localPosition.y + " " + localPosition.z);
            newEntity.setAttribute("quaternion", orientation.x + " " + orientation.y + " " + orientation.z + " " + orientation.w);
            newEntity.setAttribute("scale", scale.x + " " + scale.y + " " + scale.z);
            newEntity.setAttribute("oid", uuid.v4().toString());
            if (newEntity.flushToDOM) {
                newEntity.flushToDOM(true);
            }
            this.client.storeEntity(newEntity.outerHTML, position.x, position.y, position.z).catch(error => {
                console.error("Error saving entity", error);
            });
        }
    }
    removeEntity(entity) {
        if (this.client) {
            const entitySid = entity.getAttribute("sid");
            if (entitySid) {
                //console.log("removing from storage");
                const position = entity.object3D.position.clone();
                const worldPosition = entity.object3D.getWorldPosition(position);
                this.client.removeStoredEntity(entitySid, worldPosition.x, worldPosition.y, worldPosition.z).catch(error => {
                    //console.log("Error removing entity.", error);
                });
            }
            //console.log("removing locally");
            entity.parentElement.removeChild(entity);
        }
    }
}
SpaceSystemController.DEFINITION = new aframe_typescript_boilerplate_1.SystemControllerDefinition("space", {}, (system, scene, data) => new SpaceSystemController(system, scene, data));
exports.SpaceSystemController = SpaceSystemController;
//# sourceMappingURL=SpaceSystemController.js.map