"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
class Spring {
    constructor() {
        this.relaxationTime = 0.45;
        this.currentPosition = new three_1.Vector3();
        this.targetPosition = new three_1.Vector3();
        this.temporary = new three_1.Vector3();
        this.currentOrientation = new three_1.Quaternion();
        this.targetOrientation = new three_1.Quaternion();
    }
    simulate(t) {
        if (t > 0.04) {
            t = 0.04;
        }
        // Calculate distance between current position and target position.
        const totalDistance = this.currentPosition.distanceTo(this.targetPosition);
        let v = Math.pow(totalDistance / this.relaxationTime, 2);
        if (v < 0.3) {
            v = 0.3;
        }
        let s = v * t;
        if (s > totalDistance * 0.5) {
            s = totalDistance * 0.5;
        }
        // Calculate normalized direction vector between current position and target position.
        this.temporary.copy(this.targetPosition);
        this.temporary.sub(this.currentPosition).normalize();
        // Calculate delta vector.
        this.temporary.multiplyScalar(s);
        // Add delta to current position-
        this.currentPosition.add(this.temporary);
        // Interpolate orientation change
        this.currentOrientation.slerp(this.targetOrientation, t / this.relaxationTime);
        this.currentOrientation.normalize();
    }
}
exports.Spring = Spring;
//# sourceMappingURL=Spring.js.map