"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
const factories = new Map();
function registerStateFactory(state, factory) {
    factories.set(state, factory);
}
exports.registerStateFactory = registerStateFactory;
class StateSystemController extends aframe_typescript_boilerplate_1.AbstractSystemController {
    constructor(system, scene, data) {
        super(system, scene, data);
        this.states = new Map();
    }
    init() {
    }
    pause() {
    }
    play() {
    }
    tick(time, timeDelta) {
    }
    getState(entity, state) {
        if (!factories.has(state)) {
            throw new Error("Unknown state type: " + state);
        }
        if (!this.states.has(entity)) {
            this.states.set(entity, new Map());
        }
        const entityStates = this.states.get(entity);
        if (!entityStates.has(state)) {
            entityStates.set(state, factories.get(state)());
            //console.log(entity.tagName + " added state: " + state);
        }
        return entityStates.get(state);
    }
    removeStates(entity) {
        this.states.delete(entity);
        //console.log(entity.tagName + " removed states.");
    }
    removeState(entity, state) {
        if (this.states.has(entity)) {
            this.states.get(entity).delete(state);
        }
        //console.log(entity.tagName + " removed state: " + state);
    }
}
StateSystemController.DEFINITION = new aframe_typescript_boilerplate_1.SystemControllerDefinition("state", {}, (system, scene, data) => new StateSystemController(system, scene, data));
exports.StateSystemController = StateSystemController;
//# sourceMappingURL=StateSystemController.js.map