"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aframe_typescript_boilerplate_1 = require("aframe-typescript-boilerplate");
const three_1 = require("three");
class UiSystemController extends aframe_typescript_boilerplate_1.AbstractSystemController {
    constructor(system, scene, data) {
        super(system, scene, data);
        this.uiFrame = undefined;
        this.viewStack = new Array();
    }
    init() {
        this.uiFrame = aframe_typescript_boilerplate_1.createElement("<a-entity id='ui-frame'/>");
        this.scene.appendChild(this.uiFrame);
    }
    pause() {
    }
    play() {
    }
    tick(time, timeDelta) {
    }
    show() {
        this.uiFrame.setAttribute("visible", true);
        const interfaceSystem = this.getSystemController("interface");
        const cameraPosition = interfaceSystem.cameraPosition;
        const cameraDirection = interfaceSystem.cameraDirection;
        const uiPosition = new three_1.Vector3(cameraDirection.x, 0, cameraDirection.z);
        uiPosition.multiplyScalar(0.8);
        uiPosition.add(cameraPosition);
        const uiObject = this.uiFrame.object3D;
        const direction = new three_1.Vector3().copy(uiPosition).sub(cameraPosition).normalize();
        direction.y = 0;
        direction.normalize();
        const uiNormal = uiObject.localToWorld(new three_1.Vector3(0, 0, -1)).sub(uiObject.localToWorld(new three_1.Vector3(0, 0, 0))).normalize();
        uiNormal.y = 0;
        uiNormal.normalize();
        const quaternion = new three_1.Quaternion(); // create one and reuse it
        quaternion.setFromUnitVectors(uiNormal, direction);
        uiObject.position.copy(uiPosition);
        uiObject.quaternion.copy(quaternion.multiply(uiObject.quaternion));
    }
    hide() {
        this.uiFrame.setAttribute("visible", false);
    }
    pushView(viewTemplate) {
        const first = this.viewStack.length == 0;
        if (this.viewStack.length > 0) {
            const currentView = this.viewStack[this.viewStack.length - 1];
            this.uiFrame.removeChild(currentView);
        }
        const newView = aframe_typescript_boilerplate_1.createElement(viewTemplate);
        this.viewStack.push(newView);
        this.uiFrame.appendChild(newView);
        if (first) {
            this.show();
        }
    }
    popView() {
        if (this.viewStack.length > 0) {
            const currentView = this.viewStack.pop();
            this.uiFrame.removeChild(currentView);
        }
        if (this.viewStack.length > 0) {
            const currentView = this.viewStack[this.viewStack.length - 1];
            this.uiFrame.appendChild(currentView);
        }
        const last = this.viewStack.length == 0;
        if (last) {
            this.hide();
        }
    }
}
UiSystemController.DEFINITION = new aframe_typescript_boilerplate_1.SystemControllerDefinition("ui", {}, (system, scene, data) => new UiSystemController(system, scene, data));
exports.UiSystemController = UiSystemController;
//# sourceMappingURL=UiSystemController.js.map