"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const merge_util_1 = require("./merge_util");
const models = new Map();
const modelLoadedCallbacks = new Map();
function setEntityGltfModel(entity, src) {
    return __awaiter(this, void 0, void 0, function* () {
        const gltf = yield getGltfModel(src);
        let mesh = merge_util_1.cloneObject3D(gltf.scene) || merge_util_1.cloneObject3D(gltf.scenes[0]);
        mesh.animations = gltf.animations;
        entity.setObject3D('mesh', mesh);
        entity.emit('model-loaded', { format: 'gltf', model: mesh });
    });
}
exports.setEntityGltfModel = setEntityGltfModel;
function getGltfModel(src) {
    return new Promise(function (resolve, reject) {
        if (!models.has(src)) {
            if (!modelLoadedCallbacks.has(src)) {
                modelLoadedCallbacks.set(src, []);
                loadGltfModelQueue(src).then(() => {
                    modelLoadedCallbacks.get(src).push(() => {
                        invokeCallbackWithDelay(src, reject, resolve);
                    });
                });
            }
            else {
                modelLoadedCallbacks.get(src).push(() => {
                    invokeCallbackWithDelay(src, reject, resolve);
                });
            }
        }
        else {
            setTimeout(() => {
                resolve(models.get(src));
            }, 50);
        }
    });
}
exports.getGltfModel = getGltfModel;
function invokeCallbackWithDelay(src, reject, resolve) {
    if (!models.has(src)) {
        console.error("GLTF manager - reporting to waiter that loading failed: " + src);
        reject(new Error("Loading has failed: " + src));
    }
    else {
        setTimeout(() => {
            resolve(models.get(src));
        }, 50);
    }
}
function loadGltfModelQueue(src) {
    return __awaiter(this, void 0, void 0, function* () {
        while (!(yield attemptGltfModelWithDelay(src))) { }
    });
}
function attemptGltfModelWithDelay(src) {
    return new Promise(function (resolve, reject) {
        try {
            setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                if (loading) {
                    resolve(false);
                    return;
                }
                yield loadGltfModel(src);
                resolve(true);
            }), Math.random() * 300);
        }
        catch (error) {
            reject(error);
        }
    });
}
let loading = false;
function setMaterialAnisotropy(material) {
    if (material.map) {
        console.log('disabled mipmaps.');
        material.map.anisotropy = 1;
        material.map.generateMipmaps = false;
        material.map.minFilter = three_1.LinearFilter;
        if (material.map.image) {
            console.log('texture ' + material.map.image.name + 'format: ' + material.map.format + ' width: ' + material.map.image.width + ' height: ' + material.map.image.height + ' expected size: ' + (4 * 1.3 * (material.map.image.width * material.map.image.height) / (1024 * 1024)) + 'Mb');
        }
    }
}
function setObjectMaterialAnisotropy(object) {
    if (object.type == "Mesh") {
        const mesh = object;
        if (mesh.material instanceof Array) {
            const materials = mesh.material;
            for (const material of materials) {
                setMaterialAnisotropy(material);
            }
        }
        else if (mesh.material) {
            setMaterialAnisotropy(mesh.material);
        }
    }
    if (object.children) {
        for (const child of object.children) {
            setObjectMaterialAnisotropy(child);
        }
    }
}
function loadGltfModel(src) {
    return __awaiter(this, void 0, void 0, function* () {
        loading = true;
        const loader = new three_1.GLTFLoader();
        loader.load(src, function (gltf) {
            let scene = (gltf.scene || gltf.scenes[0]);
            setObjectMaterialAnisotropy(scene);
            //console.log("GLTF manager - loaded: " + src);
            models.set(src, gltf);
            for (const modelLoaded of modelLoadedCallbacks.get(src)) {
                modelLoaded();
            }
            modelLoadedCallbacks.delete(src);
            loading = false;
        }, undefined /* onProgress */, function (error) {
            console.error("GLTF manager - loading failed: " + src, error);
            for (const modelLoaded of modelLoadedCallbacks.get(src)) {
                modelLoaded();
            }
            modelLoadedCallbacks.delete(src);
            loading = false;
        });
    });
}
//# sourceMappingURL=gltf_load.js.map