"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = __importDefault(require("uuid"));
const config_1 = __importDefault(require("config"));
const passport_1 = __importDefault(require("passport"));
const passport_local_1 = __importDefault(require("passport-local"));
const passport_facebook_1 = __importDefault(require("passport-facebook"));
const User_1 = require("./model/User");
const util_1 = require("./util/util");
const jwt_1 = require("./util/jwt");
const log_1 = require("./util/log");
const LocalStrategy = passport_local_1.default.Strategy;
const idTokenIssuers = new Map();
const administrators = (config_1.default.get('AccessControl.administrators')).split(',');
function initializeAuthentication(app) {
    return __awaiter(this, void 0, void 0, function* () {
        (yield util_1.getIdTokenIssuers()).forEach(idTokenIssuer => {
            idTokenIssuers.set(idTokenIssuer.issuer, idTokenIssuer);
        });
        app.use(passport_1.default.initialize());
        app.use(passport_1.default.session());
        const env = process.env.NODE_ENV || 'dev';
        console.log("code reality - environment: " + env);
        // TODO disable mock login
        passport_1.default.use(new LocalStrategy({ usernameField: "username" }, function (username, password, cb) {
            if (username) {
                const userId = "an" + uuid_1.default.v4();
                const user = createFormAuthenticatedUser(userId, "Guest - " + username, "viewers");
                log_1.info(user, 'form authentication success: ' + username);
                return cb(null, user);
            }
            else {
                log_1.warnWithoutContext('form authentication success: ' + username);
                return cb(null, null);
            }
        }));
        passport_1.default.use(new passport_facebook_1.default.Strategy({
            clientID: config_1.default.get('Facebook.clientId'),
            clientSecret: config_1.default.get('Facebook.clientSecret'),
            callbackURL: config_1.default.get('Facebook.callbackUrl'),
            profileFields: ['id', 'name', 'displayName']
        }, (accessToken, refreshToken, profile, cb) => {
            const userId = "fb" + profile.id;
            let userName = profile.displayName;
            let userGroups = "users";
            if (administrators.length > 0 && administrators[0].length > 0 && administrators.indexOf(userId) > -1) {
                userGroups += ",administrators";
                userName = "Admin - " + userName;
            }
            let user = createFormAuthenticatedUser(userId, userName, userGroups);
            return cb(null, user);
        }));
        passport_1.default.serializeUser(function (user, cb) {
            cb(null, user);
        });
        passport_1.default.deserializeUser(function (obj, cb) {
            cb(null, obj);
        });
        app.use(function (req, res, next) {
            const requestId = req.headers['request-id'];
            const user = req.user;
            if (user) {
                user.requestId = requestId;
            }
            const authenticated = user !== null && user !== undefined;
            const bearerToken = req.headers.authorization && req.headers.authorization.startsWith("Bearer ");
            // No authentication for the following paths.
            if (req.path.startsWith("/css/") ||
                req.path.startsWith("/images/") ||
                req.path.startsWith("/models/") ||
                req.path.startsWith("/api/auth") ||
                req.path === "/login" ||
                req.path === "/login.html" ||
                req.path === "/login_failed.html" ||
                req.path === "/https_required.html" ||
                req.path === "/health" ||
                req.path === "/favicon.ico") {
                return next();
            }
            // JWT authentication if bearer token exists.
            if (bearerToken) {
                log_1.infoWithRequestId(requestId, "authenticating with bearer token and serving.");
                return bearerTokenAuthentication(req, res, next);
            }
            // Form authentication for rest of the cases.
            if (authenticated) {
                return next();
            }
            else {
                log_1.infoWithRequestId(requestId, "redirecting to form login.");
                req.session.afterLoginUrl = req.path;
                res.redirect('/login.html');
            }
        });
        app.post('/login', passport_1.default.authenticate('local', { session: true, failureRedirect: '/login_failed.html' }), function (req, res) {
            afterLoginRedirect(req, res);
        });
        app.get('/api/auth/facebook', passport_1.default.authenticate('facebook', {}));
        app.get('/api/auth/facebook/callback', passport_1.default.authenticate('facebook', { failureRedirect: '/api/login_failed.html' }), (req, res) => {
            afterLoginRedirect(req, res);
        });
    });
}
exports.initializeAuthentication = initializeAuthentication;
;
function afterLoginRedirect(req, res) {
    const user = req.user;
    log_1.info(user, "User authenticated: " + user.userId + " : " + user.userName);
    if (req.session.afterLoginUrl) {
        res.redirect(req.session.afterLoginUrl);
    }
    else {
        res.redirect('/');
    }
}
function bearerTokenAuthentication(req, res, next) {
    try {
        if (!req.headers.authorization) {
            return respondError(req, res, 401, "authorization header does not exist.", undefined);
        }
        const idToken = req.headers.authorization.substring("Bearer ".length);
        doJwtAuthentication(req, res, idToken, next);
    }
    catch (error) {
        return respondError(req, res, 401, "access denied", error);
    }
}
function doJwtAuthentication(req, res, idToken, next) {
    try {
        const requestId = req.headers['request-id'];
        if (!requestId) {
            return respondError(req, res, 401, "request-ID header does not exist.", undefined);
        }
        const issuer = jwt_1.decodeIdToken(idToken).get("iss");
        if (!issuer) {
            return respondError(req, res, 401, "issuer claim not found.", undefined);
        }
        const idTokenIssuer = idTokenIssuers.get(issuer);
        if (!idTokenIssuer) {
            return respondError(req, res, 401, "issuer not found from configuration: " + issuer, undefined);
        }
        const claims = jwt_1.validateIdToken(idToken, idTokenIssuer.publicKey);
        if (!claims.has("id") || !claims.has("exp") || !claims.has("name") || !claims.get("jti")) {
            return respondError(req, res, 401, "missing mandatory claims.", undefined);
        }
        const tokenId = jwt_1.decodeIdToken(idToken).get("jti");
        const userId = claims.get("id");
        const userName = claims.get("name");
        const groupsString = claims.get("groups");
        const groups = groupsString ? groupsString.split(",") : undefined;
        const user = new User_1.User(idToken, tokenId, requestId, userId, userName, groups, issuer);
        req.login(user, function (err) {
            if (err) {
                return next(err);
            }
            next();
        });
    }
    catch (error) {
        return respondError(req, res, 401, "access denied", error);
    }
}
function createFormAuthenticatedUser(userId, username, groups) {
    const requestId = uuid_1.default.v4().toString();
    const issuer = config_1.default.get("IdToken.issuer");
    const privateKeyEncoded = config_1.default.get("IdToken.privateKey");
    const privateKeyPassword = config_1.default.get("IdToken.privateKeyPassword");
    const groupsArray = groups.split(',');
    const idToken = jwt_1.createIdToken(issuer, userId, username, groups, privateKeyEncoded, privateKeyPassword);
    const tokenId = jwt_1.decodeIdToken(idToken).get("jti");
    const user = new User_1.User(idToken, tokenId, requestId, userId, username, groupsArray, issuer);
    return user;
}
function respondError(req, res, status, message, err) {
    const requestId = req.headers['request-id'];
    const user = req.user;
    if (err) {
        if (user) {
            log_1.error(user, message, err);
        }
        else if (requestId) {
            log_1.errorWithRequestId(requestId, message, err);
        }
        else {
            log_1.errorWithoutContext(message, err);
        }
    }
    else {
        if (user) {
            log_1.warn(user, message);
        }
        else if (requestId) {
            log_1.warnWithRequestId(requestId, message);
        }
        else {
            log_1.warnWithoutContext(message);
        }
    }
    return res.status(status).send(message);
}
//# sourceMappingURL=authentication.js.map