"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const config = require("config");
const log_1 = require("./util/log");
const BrowserContext_1 = require("../common/model/BrowserContext");
const fs = require('fs');
function initializeRoutes(app) {
    app.get('/api/context', function (request, response) {
        const requestId = request.headers['request-id'];
        //infoWithRequestId(requestId, '/api/users/current: ' + JSON.stringify(request.user));
        const user = request.user;
        response.send(JSON.stringify(new BrowserContext_1.BrowserContext(user.userId, user.userName, user.groups ? user.groups : [], user.idToken, config.get("Cluster.configurationUrl"), request.session.space ? request.session.space : "default")));
    });
    app.get('/api/users/current', function (request, response) {
        const requestId = request.headers['request-id'];
        log_1.infoWithRequestId(requestId, '/api/users/current: ' + JSON.stringify(request.user));
        response.send(JSON.stringify(request.user));
    });
    app.get('/api/users/current/id-token', function (request, response) {
        const requestId = request.headers['request-id'];
        log_1.infoWithRequestId(requestId, '/api/users/current/id-token: ' + JSON.stringify(request.user));
        response.send(request.user.idToken);
    });
    app.get('/api/users/current/space', function (request, response) {
        const requestId = request.headers['request-id'];
        if (request.session.space) {
            log_1.infoWithRequestId(requestId, '/api/users/current/space: ' + request.session.space);
            response.send(request.session.space);
        }
        else {
            log_1.infoWithRequestId(requestId, '/api/users/current/space: ' + "default");
            response.send("default");
        }
    });
    app.get('/api/users/current/cluster-url', function (request, response) {
        response.send(config.get("Cluster.configurationUrl"));
    });
    app.get('/health', function (request, response) {
        response.status(200);
        response.send('');
    });
    app.get("/:space", function (request, response) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestId = request.headers['request-id'];
            const space = request.params.space;
            log_1.infoWithRequestId(requestId, "set current space according to query path to: " + space);
            request.session.space = space;
            if (fs.existsSync('./static/index.html')) {
                response.sendFile('index.html', { root: './static' });
            }
            else {
                response.sendFile('index.html', { root: './dist' });
            }
        });
    });
}
exports.initializeRoutes = initializeRoutes;
//# sourceMappingURL=routes.js.map