"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const session_1 = require("./session");
const authentication_1 = require("./authentication");
const routes_1 = require("./routes");
const express_1 = __importDefault(require("express"));
const uuid = require("uuid");
const log_1 = require("./util/log");
require('console-stamp')(console, {
    pattern: 'UTC:yyyy-mm-dd\'T\'HH:MM:ss\'Z\'',
    colors: { stamp: 'yellow', label: 'white', metadata: 'green' }
});
function newServer(host, port) {
    return __awaiter(this, void 0, void 0, function* () {
        const app = express_1.default();
        app.use(requireHttpsMiddleware);
        app.use(requestIdAndLoggingMiddleware);
        session_1.initializeSession(app);
        app.use(spaceSelectionMiddleware);
        yield authentication_1.initializeAuthentication(app);
        /*app.get("/", async function (request, response) {
            const space = (request as any).session.space ? (request as any).session.space : 'default';
            if (request.query.space) {
                response.redirect("/" + request.query.space);
            }
        });*/
        app.use(express_1.default.static('static'));
        app.use(express_1.default.static('dist'));
        routes_1.initializeRoutes(app);
        const server = app.listen(port, host, function () {
            console.log('code-reality - http server listening at local URL: http://' + host + ':' + port + '/');
        });
        return server;
    });
}
exports.newServer = newServer;
function requireHttpsMiddleware(req, res, next) {
    const env = process.env.NODE_ENV || 'dev';
    if (!env.startsWith('dev') && !req.path.startsWith("/https_required.html") && !req.path.startsWith("/css/")) {
        if (req.headers['x-forwarded-proto']) {
            if (req.headers['x-forwarded-proto'] !== 'https') {
                return res.redirect("/https_required.html");
            }
        }
        else if (req.protocol !== 'https') {
            return res.redirect("/https_required.html");
        }
    }
    // allow the request to continue
    next();
}
function requestIdAndLoggingMiddleware(req, res, next) {
    const oldEnd = res.end;
    // Add request-id header to request if it does not exist.
    if (!req.headers['request-id']) {
        req.headers['request-id'] = uuid.v4().toString();
    }
    // Log request response status code.
    res.end = function (...args) {
        const requestId = req.headers['request-id'];
        const user = req.user;
        const message = res.statusCode + " " + req.method + " " + req.url;
        if (user) {
            log_1.info(user, message);
        }
        else if (requestId) {
            log_1.infoWithRequestId(requestId, message);
        }
        else {
            log_1.infoWithoutContext(message);
        }
        oldEnd.apply(res, args);
    };
    next();
}
function spaceSelectionMiddleware(req, res, next) {
    if (req.query.space) {
        const requestId = req.headers['request-id'];
        log_1.infoWithRequestId(requestId, "set current space according to query parameter to: " + req.query.space);
        //(req as any).session.space = req.query.space;
        res.redirect("/" + req.query.space);
    }
    else {
        if (req.path === "/") {
            req.session.space = "default";
        }
        next();
    }
}
//# sourceMappingURL=server.js.map