"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jwt = require('jsonwebtoken');
const uuid = require("uuid");
function createIdToken(issuer, userId, userName, groups, privateKeyEncoded, privateKeyPassword) {
    let privateKey;
    if (privateKeyEncoded.startsWith('-')) {
        privateKey = privateKeyEncoded;
    }
    else {
        privateKey = Buffer.from(privateKeyEncoded, 'base64').toString();
    }
    var token = jwt.sign({
        iss: issuer,
        id: userId,
        jti: uuid.v4(),
        name: userName,
        groups: groups,
        exp: Math.floor(Date.now() / 1000) + (8 * 60 * 60),
    }, {
        key: privateKey,
        passphrase: privateKeyPassword
    }, {
        algorithm: 'RS256'
    });
    return token;
}
exports.createIdToken = createIdToken;
function validateIdToken(idToken, publicKeyEncoded) {
    let publicKey;
    if (publicKeyEncoded.startsWith('-')) {
        publicKey = publicKeyEncoded;
    }
    else {
        publicKey = Buffer.from(publicKeyEncoded, 'base64').toString();
    }
    var decoded = jwt.verify(idToken, publicKey, { algorithm: 'RS256' });
    const map = new Map();
    Object.keys(decoded).forEach(key => {
        map.set(key, decoded[key]);
    });
    return map;
}
exports.validateIdToken = validateIdToken;
function decodeIdToken(idToken) {
    var decoded = jwt.decode(idToken);
    const map = new Map();
    Object.keys(decoded).forEach(key => {
        map.set(key, decoded[key]);
    });
    return map;
}
exports.decodeIdToken = decodeIdToken;
//# sourceMappingURL=jwt.js.map